"""Domain 层仓储实现 - BaseRepository 的具体实现。

提供通用的 CRUD 操作实现，是 IRepository 接口的具体实现。

**重要架构决策**：本模块在 domain 层实现了 IRepository 接口。
Infrastructure 层现在仅负责数据库连接管理，完全不依赖 domain 层。
"""

from __future__ import annotations

import time
from typing import TYPE_CHECKING, Any, Union, cast

from sqlalchemy import Select, delete, func, select
from sqlalchemy.ext.asyncio import AsyncSession

from aury.boot.common.logging import logger
from aury.boot.domain.exceptions import VersionConflictError
from aury.boot.domain.models import GUID, Base
from aury.boot.domain.pagination import (
    CursorPaginationParams,
    CursorPaginationResult,
    PaginationParams,
    PaginationResult,
    SortParams,
)
from aury.boot.domain.repository.interface import IRepository
from aury.boot.domain.repository.query_builder import QueryBuilder
from aury.boot.domain.transaction import _transaction_depth

if TYPE_CHECKING:
    from typing import Self


def _get_model_attr(model_class: type, attr_name: str) -> Any:
    """安全地获取模型类的属性，避免类型检查器警告。
    
    Args:
        model_class: SQLAlchemy 模型类
        attr_name: 属性名称
        
    Returns:
        属性值（如果存在），否则返回 None
    """
    return cast(Any, getattr(model_class, attr_name, None))


class BaseRepository[ModelType: Base](IRepository[ModelType]):
    """仓储基类实现。提供通用 CRUD 操作。
    
    **重要**：使用此类的模型必须至少继承以下 Mixin 之一：
    - IDMixin: 标准自增整数主键
    - UUIDMixin: UUID 主键
    
    可选 Mixin：
    - AuditableStateMixin: 软删除支持
    - VersionMixin: 乐观锁支持
    - TimestampMixin: 时间戳字段
    
    自动提交行为：
    - auto_commit=True（默认）：非事务中自动 commit
    - auto_commit=False：只 flush，需要手动管理事务或使用 .with_commit()
    - 在事务中（transactional_context 等）：永远不自动 commit，由事务统一管理
    
    示例：
        class User(IDMixin, TimestampMixin, AuditableStateMixin, Base):
            __tablename__ = "users"
            name: Mapped[str]
        
        # 默认 auto_commit=True，非事务中自动提交
        repo = UserRepository(session, User)
        await repo.create({"name": "test"})  # 自动 commit
        
        # auto_commit=False，需要手动管理
        repo = UserRepository(session, User, auto_commit=False)
        await repo.create({"name": "test"})  # 只 flush
        await repo.with_commit().create({"name": "test2"})  # 强制 commit
    """
    
    def __init__(
        self,
        session: AsyncSession,
        model_class: type[ModelType],
        auto_commit: bool = True,
    ) -> None:
        self._session = session
        self._model_class = model_class
        self._auto_commit = auto_commit
        self._force_commit = False  # 单次强制提交标记（用于 with_commit）
        logger.debug(f"初始化 {self.__class__.__name__}")
    
    @property
    def session(self) -> AsyncSession:
        return self._session
    
    @property
    def model_class(self) -> type[ModelType]:
        return self._model_class
    
    @property
    def auto_commit(self) -> bool:
        """是否启用自动提交。"""
        return self._auto_commit
    
    def with_commit(self) -> Self:
        """返回强制提交的 Repository 视图。
        
        在 auto_commit=False 时，使用此方法可以强制单次操作提交。
        
        示例：
            repo = UserRepository(session, User, auto_commit=False)
            await repo.create({"name": "test"})  # 只 flush
            await repo.with_commit().create({"name": "test2"})  # 强制 commit
        
        Returns:
            Self: 带强制提交标记的 Repository 副本
        """
        clone = self.__class__(
            self._session,
            self._model_class,
            self._auto_commit,
        )
        clone._force_commit = True
        return clone
    
    async def _maybe_commit(self) -> None:
        """根据配置决定是否提交。
        
        提交条件：
        1. 不在显式事务中（transactional_context / @transactional）
        2. 满足以下任一条件：
           - _force_commit=True（来自 with_commit()）
           - _auto_commit=True（默认配置）
        
        注意：使用 _transaction_depth 而非 session.in_transaction() 判断，
        因为 SQLAlchemy 的 autobegin 会让 in_transaction() 在任何 SQL 执行后返回 True，
        但这不代表用户在显式事务中。
        """
        # 在显式事务中（transactional_context / @transactional），由事务统一管理
        if _transaction_depth.get() > 0:
            return
        
        # 强制提交或自动提交
        if self._force_commit or self._auto_commit:
            await self._session.commit()
            logger.debug("Repository 自动提交")
    
    def query(self) -> QueryBuilder[ModelType]:
        builder = QueryBuilder(self._model_class)
        if hasattr(self._model_class, "deleted_at"):
            deleted_at = _get_model_attr(self._model_class, "deleted_at")
            builder.filter_by(deleted_at == 0)
        return builder
    
    def _build_base_query(self) -> Select:
        query = select(self._model_class)
        if hasattr(self._model_class, "deleted_at"):
            deleted_at = _get_model_attr(self._model_class, "deleted_at")
            query = query.where(deleted_at == 0)
        return query
    
    def _apply_filters(self, query: Select, **filters) -> Select:
        """将简单和带操作符的 filters 应用于查询。
        
        支持键名后缀操作符（与 QueryBuilder.filter 一致）：
        - __gt, __lt, __gte, __lte, __in, __like, __ilike, __isnull, __ne
        例如：
        - name__ilike="%foo%"
        - age__gte=18
        - id__in=[1,2,3]
        - deleted_at__isnull=True
        
        注意：此处所有条件均以 AND 组合。更复杂的 AND/OR/NOT 组合请使用 repo.query()。
        """
        for key, value in filters.items():
            if value is None:
                continue
            
            if "__" in key:
                field_name, operator = key.rsplit("__", 1)
                if not hasattr(self._model_class, field_name):
                    continue
                field = _get_model_attr(self._model_class, field_name)
                
                if operator == "isnull":
                    condition = field.is_(None) if bool(value) else field.isnot(None)
                elif operator == "in":
                    condition = field.in_(value)
                elif operator == "gt":
                    condition = field > value
                elif operator == "lt":
                    condition = field < value
                elif operator == "gte":
                    condition = field >= value
                elif operator == "lte":
                    condition = field <= value
                elif operator == "like":
                    condition = field.like(value)
                elif operator == "ilike":
                    condition = field.ilike(value)
                elif operator == "ne":
                    condition = field != value
                else:
                    # 未知操作符，忽略
                    continue
                query = query.where(condition)
            else:
                if hasattr(self._model_class, key):
                    query = query.where(getattr(self._model_class, key) == value)
        return query
    
    async def get(self, id: Union[int, GUID]) -> ModelType | None:
        """按 ID 获取实体，支持 int 和 GUID。"""
        if not hasattr(self._model_class, "id"):
            raise AttributeError(f"模型 {self._model_class.__name__} 没有 'id' 字段，请继承 IDMixin 或 UUIDMixin")
        id_attr = _get_model_attr(self._model_class, "id")
        query = self._build_base_query().where(id_attr == id)
        result = await self._session.execute(query)
        return result.scalar_one_or_none()
    
    async def get_by(self, **filters) -> ModelType | None:
        query = self._build_base_query()
        query = self._apply_filters(query, **filters)
        result = await self._session.execute(query)
        return result.scalar_one_or_none()
    
    async def list(
        self,
        skip: int = 0,
        limit: int | None = None,
        sort: str | SortParams | list[str] | None = None,
        **filters
    ) -> list[ModelType]:
        """获取实体列表。
        
        Args:
            skip: 跳过记录数
            limit: 返回记录数限制，None 表示不限制（默认）
            sort: 排序参数，支持多种格式：
                - 字符串: "-created_at" 或 "created_at:desc" 或 "-created_at,name"
                - SortParams 对象
                - 字符串列表: ["-created_at", "name"]
            **filters: 过滤条件
            
        Returns:
            list[ModelType]: 实体列表
        """
        query = self._build_base_query()
        query = self._apply_filters(query, **filters)
        query = self._apply_sort(query, sort)
        query = query.offset(skip)
        if limit is not None:
            query = query.limit(limit)
        result = await self._session.execute(query)
        return list(result.scalars().all())
    
    def _apply_sort(
        self,
        query: Select,
        sort: str | SortParams | list[str] | None,
    ) -> Select:
        """应用排序参数到查询。
        
        Args:
            query: SQLAlchemy 查询对象
            sort: 排序参数，支持：
                - 字符串: "-created_at" 或 "created_at:desc" 或 "-created_at,name"
                - SortParams 对象
                - 字符串列表: ["-created_at", "name"]
                
        Returns:
            Select: 应用排序后的查询对象
        """
        if sort is None:
            return query
        
        # 统一转换为 SortParams
        sort_params: SortParams
        if isinstance(sort, SortParams):
            sort_params = sort
        elif isinstance(sort, str):
            sort_params = SortParams.from_string(sort)
        elif isinstance(sort, list):
            # 列表格式: ["-created_at", "name"]
            sort_params = SortParams.from_string(",".join(sort))
        else:
            return query
        
        order_by_list = []
        for field, direction in sort_params.sorts:
            field_attr = getattr(self._model_class, field, None)
            if field_attr is not None:
                if direction == "desc":
                    order_by_list.append(field_attr.desc())
                else:
                    order_by_list.append(field_attr)
        
        if order_by_list:
            query = query.order_by(*order_by_list)
        
        return query
    
    async def paginate(
        self,
        pagination: PaginationParams,
        sort: str | SortParams | list[str] | None = None,
        **filters
    ) -> PaginationResult[ModelType]:
        """分页查询（list 的语法糖）。
        
        Args:
            pagination: 分页参数
            sort: 排序参数，支持多种格式：
                - 字符串: "-created_at" 或 "created_at:desc" 或 "-created_at,name"
                - SortParams 对象
                - 字符串列表: ["-created_at", "name"]
            **filters: 过滤条件
            
        Returns:
            PaginationResult[ModelType]: 分页结果
        """
        items = await self.list(
            skip=pagination.offset,
            limit=pagination.limit,
            sort=sort,
            **filters
        )
        total = await self.count(**filters)
        
        return PaginationResult.create(
            items=items,
            total=total,
            pagination=pagination,
        )
    
    async def cursor_paginate(
        self,
        params: CursorPaginationParams,
        cursor_field: str = "id",
        **filters
    ) -> CursorPaginationResult[ModelType]:
        """基于游标的分页查询。
        
        适用于无限滚动、大数据集等场景，性能优于 offset 分页。
        
        Args:
            params: 游标分页参数（cursor, limit, direction）
            cursor_field: 游标字段名，默认 "id"，必须是有序且唯一的字段
            **filters: 过滤条件
            
        Returns:
            CursorPaginationResult: 包含 items, next_cursor, prev_cursor, has_next, has_prev
            
        示例:
            # 第一页
            result = await repo.cursor_paginate(
                CursorPaginationParams(limit=20),
                status="active"
            )
            
            # 下一页
            result = await repo.cursor_paginate(
                CursorPaginationParams(cursor=result.next_cursor, limit=20),
                status="active"
            )
        """
        import base64
        import json
        
        query = self._build_base_query()
        query = self._apply_filters(query, **filters)
        
        cursor_attr = _get_model_attr(self._model_class, cursor_field)
        if cursor_attr is None:
            raise AttributeError(f"模型 {self._model_class.__name__} 没有 '{cursor_field}' 字段")
        
        # 解码 cursor
        cursor_value = None
        if params.cursor:
            try:
                decoded = base64.urlsafe_b64decode(params.cursor.encode()).decode()
                cursor_value = json.loads(decoded)
            except Exception:
                raise ValueError("无效的游标") from None
        
        # 根据方向决定查询条件和排序
        is_next = params.direction == "next"
        if cursor_value is not None:
            if is_next:
                query = query.where(cursor_attr > cursor_value)
            else:
                query = query.where(cursor_attr < cursor_value)
        
        # 排序
        if is_next:
            query = query.order_by(cursor_attr)
        else:
            query = query.order_by(cursor_attr.desc())
        
        # 多取一条判断是否有更多
        query = query.limit(params.limit + 1)
        result = await self._session.execute(query)
        items = list(result.scalars().all())
        
        has_more = len(items) > params.limit
        if has_more:
            items = items[:params.limit]
        
        # 反向查询时反转结果
        if not is_next:
            items = list(reversed(items))
        
        # 生成游标
        def encode_cursor(value) -> str:
            return base64.urlsafe_b64encode(json.dumps(value).encode()).decode()
        
        next_cursor = None
        prev_cursor = None
        
        if items:
            last_value = getattr(items[-1], cursor_field)
            first_value = getattr(items[0], cursor_field)
            
            if is_next:
                if has_more:
                    next_cursor = encode_cursor(last_value)
                if cursor_value is not None:
                    prev_cursor = encode_cursor(first_value)
            else:
                if cursor_value is not None:
                    next_cursor = encode_cursor(last_value)
                if has_more:
                    prev_cursor = encode_cursor(first_value)
        
        return CursorPaginationResult.create(
            items=items,
            next_cursor=next_cursor,
            prev_cursor=prev_cursor,
            limit=params.limit,
        )
    
    async def stream(
        self,
        batch_size: int = 1000,
        sort: str | SortParams | list[str] | None = None,
        **filters
    ):
        """流式查询，使用数据库原生 server-side cursor。
        
        适用于大数据集处理，避免一次性加载所有数据到内存。
        
        Args:
            batch_size: 每批次获取的记录数，默认 1000
            sort: 排序参数（同 list）
            **filters: 过滤条件
            
        Yields:
            ModelType: 模型实例
            
        示例:
            async for user in repo.stream(batch_size=500, sort="-created_at", status="active"):
                process(user)
        """
        query = self._build_base_query()
        query = self._apply_filters(query, **filters)
        query = self._apply_sort(query, sort)
        
        async with self._session.stream_scalars(
            query.execution_options(yield_per=batch_size)
        ) as result:
            async for item in result:
                yield item
    
    async def stream_batches(
        self,
        batch_size: int = 1000,
        sort: str | SortParams | list[str] | None = None,
        **filters
    ):
        """批量流式查询，每次返回一批数据。
        
        Args:
            batch_size: 每批次的记录数，默认 1000
            sort: 排序参数（同 list）
            **filters: 过滤条件
            
        Yields:
            list[ModelType]: 一批模型实例
            
        示例:
            async for batch in repo.stream_batches(batch_size=500, sort="id"):
                bulk_process(batch)
        """
        query = self._build_base_query()
        query = self._apply_filters(query, **filters)
        query = self._apply_sort(query, sort)
        
        async with self._session.stream_scalars(
            query.execution_options(yield_per=batch_size)
        ) as result:
            async for partition in result.partitions():
                yield list(partition)
    
    async def count(self, **filters) -> int:
        query = select(func.count()).select_from(self._model_class)
        if hasattr(self._model_class, "deleted_at"):
            deleted_at = _get_model_attr(self._model_class, "deleted_at")
            query = query.where(deleted_at == 0)
        query = self._apply_filters(query, **filters)
        result = await self._session.execute(query)
        return result.scalar_one()
    
    async def exists(self, **filters) -> bool:
        """检查是否存在匹配的记录（比 count > 0 更高效）。"""
        query = self._apply_filters(self._build_base_query(), **filters).limit(1)
        result = await self._session.execute(query)
        return result.scalar_one_or_none() is not None
    
    async def add(self, entity: ModelType) -> ModelType:
        self._session.add(entity)
        await self._session.flush()
        await self._session.refresh(entity)
        await self._maybe_commit()
        logger.debug(f"添加实体: {entity}")
        return entity
    
    async def create(self, data: dict[str, Any]) -> ModelType:
        entity = self._model_class(**data)
        # add() 已调用 _maybe_commit，此处无需重复
        return await self.add(entity)
    
    async def update(self, entity: ModelType, data: dict[str, Any] | None = None) -> ModelType:
        if hasattr(entity, "version"):
            entity_any = cast(Any, entity)
            current_version = entity_any.version
            await self._session.refresh(entity, ["version"])
            if entity_any.version != current_version:
                raise VersionConflictError(
                    message="数据已被其他操作修改，请刷新后重试",
                    current_version=entity_any.version,
                    expected_version=current_version,
                )
            entity_any.version = entity_any.version + 1
        
        if data:
            for key, value in data.items():
                if hasattr(entity, key) and key != "version":
                    setattr(entity, key, value)
        
        await self._session.flush()
        await self._session.refresh(entity)
        await self._maybe_commit()
        logger.debug(f"更新实体: {entity}")
        return entity
    
    async def delete(self, entity: ModelType, soft: bool = True) -> None:
        if soft:
            if hasattr(entity, "deleted_at"):
                # 使用 setattr 因为类型检查器不知道动态属性
                setattr(entity, "deleted_at", int(time.time()))  # noqa: B010
                await self._session.flush()
                await self._maybe_commit()
                logger.debug(f"软删除实体: {entity}")
            elif hasattr(entity, "mark_deleted"):
                entity.mark_deleted()
                await self._session.flush()
                await self._maybe_commit()
                logger.debug(f"软删除实体（使用方法）: {entity}")
            else:
                await self._session.delete(entity)
                await self._session.flush()
                await self._maybe_commit()
                logger.debug(f"硬删除实体（无软删除字段）: {entity}")
        else:
            await self._session.delete(entity)
            await self._session.flush()
            await self._maybe_commit()
            logger.debug(f"硬删除实体: {entity}")
    
    async def mark_deleted(self, entity: ModelType) -> None:
        await self.delete(entity, soft=True)
    
    async def hard_delete(self, entity: ModelType) -> None:
        await self.delete(entity, soft=False)
    
    async def delete_by_id(self, id: int, soft: bool = True) -> bool:
        entity = await self.get(id)
        if entity:
            await self.delete(entity, soft=soft)
            return True
        return False
    
    async def batch_create(self, data_list: list[dict[str, Any]]) -> list[ModelType]:
        entities = [self._model_class(**data) for data in data_list]
        self._session.add_all(entities)
        await self._session.flush()
        for entity in entities:
            await self._session.refresh(entity)
        await self._maybe_commit()
        logger.debug(f"批量创建 {len(entities)} 个实体")
        return entities
    
    async def bulk_insert(self, data_list: list[dict[str, Any]]) -> None:
        if not data_list:
            return
        await self._session.execute(
            self._model_class.__table__.insert(),
            data_list
        )
        await self._session.flush()
        await self._maybe_commit()
        logger.debug(f"批量插入 {len(data_list)} 条记录")
    
    async def bulk_update(self, data_list: list[dict[str, Any]],
                         index_elements: list[str] | None = None) -> None:
        if not data_list:
            return
        if index_elements is None:
            index_elements = ["id"]
        for data in data_list:
            for field in index_elements:
                if field not in data:
                    raise ValueError(f"批量更新数据缺少索引字段: {field}")
        await self._session.bulk_update_mappings(self._model_class, data_list)
        await self._session.flush()
        await self._maybe_commit()
        logger.debug(f"批量更新 {len(data_list)} 条记录")
    
    async def bulk_delete(self, filters: dict[str, Any] | None = None) -> int:
        query = delete(self._model_class)
        if hasattr(self._model_class, "deleted_at"):
            deleted_at = _get_model_attr(self._model_class, "deleted_at")
            query = query.where(deleted_at == 0)
        if filters:
            for key, value in filters.items():
                if hasattr(self._model_class, key) and value is not None:
                    attr = _get_model_attr(self._model_class, key)
                    query = query.where(attr == value)
        result = await self._session.execute(query)
        await self._session.flush()
        await self._maybe_commit()
        deleted_count = cast(Any, result).rowcount
        logger.debug(f"批量删除 {deleted_count} 条记录")
        return deleted_count
    
    def __repr__(self) -> str:
        return f"<{self.__class__.__name__} model={self._model_class.__name__}>"


class SimpleRepository:
    """简单Repository基类。"""
    
    def __init__(self, session: AsyncSession) -> None:
        self._session = session
    
    @property
    def session(self) -> AsyncSession:
        return self._session
    
    def __repr__(self) -> str:
        return f"<{self.__class__.__name__}>"


__all__ = ["BaseRepository", "SimpleRepository"]



"""任务队列相关异常定义。

Infrastructure 层异常，继承自 FoundationError。
"""

from __future__ import annotations

from aury.boot.common.exceptions import FoundationError


class TaskError(FoundationError):
    """任务队列相关错误基类。
    
    所有任务队列相关的异常都应该继承此类。
    """
    
    pass


class TaskQueueError(TaskError):
    """任务队列错误。"""
    
    pass


class TaskExecutionError(TaskError):
    """任务执行错误。"""
    
    pass


__all__ = [
    "TaskError",
    "TaskExecutionError",
    "TaskQueueError",
]




"""Broadcaster 事件总线后端。

使用 broadcaster 库实现事件发布/订阅，支持多种后端：
- memory://  内存（单进程）
- redis://   Redis Pub/Sub（多进程/多实例）
- kafka://   Apache Kafka
- postgres:// PostgreSQL LISTEN/NOTIFY
"""

from __future__ import annotations

import asyncio
import json

from broadcaster import Broadcast

from aury.boot.common.logging import logger

from ..base import Event, EventHandler, IEventBus

# 框架默认前缀
DEFAULT_CHANNEL_PREFIX = "aury:event:"


class BroadcasterEventBus(IEventBus):
    """Broadcaster 事件总线实现。

    使用 broadcaster 库实现事件发布/订阅。
    
    频道命名格式：{channel_prefix}{event_name}
    默认：aury:event:user.created
    
    优点：
    - 统一接口支持多种后端
    - 内置连接池管理
    - 自动重连机制
    """

    def __init__(
        self,
        url: str,
        *,
        channel_prefix: str = DEFAULT_CHANNEL_PREFIX,
    ) -> None:
        """初始化 Broadcaster 事件总线。

        Args:
            url: 连接 URL，格式：
                - memory://          内存（单进程）
                - redis://host:port  Redis Pub/Sub
                - kafka://host:port  Apache Kafka
                - postgres://...     PostgreSQL
            channel_prefix: 频道名称前缀，默认 "aury:event:"

        """
        self._url = url
        self._channel_prefix = channel_prefix
        self._broadcast: Broadcast | None = None
        # event_name -> list of handlers (本地订阅)
        self._handlers: dict[str, list[EventHandler]] = {}
        self._listener_tasks: dict[str, asyncio.Task] = {}
        self._running = False

    async def _ensure_connected(self) -> None:
        """确保已连接。"""
        if self._broadcast is None:
            self._broadcast = Broadcast(self._url)
            await self._broadcast.connect()
            logger.debug(f"Broadcaster 事件总线已连接: {self._url}")

    def _get_event_name(self, event_type: type[Event] | str) -> str:
        """获取事件名称。"""
        if isinstance(event_type, str):
            return event_type
        return event_type.__name__

    def _get_channel(self, event_name: str) -> str:
        """获取频道名称。"""
        return f"{self._channel_prefix}{event_name}"

    def subscribe(
        self,
        event_type: type[Event] | str,
        handler: EventHandler,
    ) -> None:
        """订阅事件。

        注意：这是同步注册 handler，真正的监听在 start_listening() 中启动。
        """
        event_name = self._get_event_name(event_type)
        if event_name not in self._handlers:
            self._handlers[event_name] = []
        if handler not in self._handlers[event_name]:
            self._handlers[event_name].append(handler)
            logger.debug(f"订阅事件: {event_name} -> {handler.__name__}")

            # 如果已经在运行，立即为新事件启动监听
            if self._running and event_name not in self._listener_tasks:
                task = asyncio.create_task(self._listen_event(event_name))
                self._listener_tasks[event_name] = task

    def unsubscribe(
        self,
        event_type: type[Event] | str,
        handler: EventHandler,
    ) -> None:
        """取消订阅事件。"""
        event_name = self._get_event_name(event_type)
        if event_name in self._handlers:
            try:
                self._handlers[event_name].remove(handler)
                logger.debug(f"取消订阅事件: {event_name} -> {handler.__name__}")
                
                # 如果该事件没有处理器了，停止监听
                if not self._handlers[event_name] and event_name in self._listener_tasks:
                    self._listener_tasks[event_name].cancel()
                    del self._listener_tasks[event_name]
            except ValueError:
                pass

    async def publish(self, event: Event) -> None:
        """发布事件。"""
        await self._ensure_connected()
        event_name = event.event_name
        channel = self._get_channel(event_name)
        data = json.dumps(event.to_dict())
        await self._broadcast.publish(channel=channel, message=data)

    async def _listen_event(self, event_name: str) -> None:
        """监听单个事件的消息。"""
        channel = self._get_channel(event_name)
        try:
            async with self._broadcast.subscribe(channel=channel) as subscriber:
                async for event_data in subscriber:
                    if not self._running:
                        break
                    try:
                        data = json.loads(event_data.message)
                        handlers = self._handlers.get(event_name, [])
                        for handler in handlers:
                            try:
                                event = Event.from_dict(data)
                                result = handler(event)
                                if asyncio.iscoroutine(result):
                                    await result
                            except Exception as e:
                                logger.error(f"处理事件 {event_name} 失败: {e}")
                    except (json.JSONDecodeError, KeyError) as e:
                        logger.warning(f"解析事件消息失败: {e}")
        except asyncio.CancelledError:
            pass
        except Exception as e:
            logger.error(f"事件监听异常 {event_name}: {e}")

    async def start_listening(self) -> None:
        """开始监听事件（需要在后台任务中运行）。"""
        if self._running:
            return

        await self._ensure_connected()
        self._running = True

        # 为每个已订阅的事件启动监听任务
        for event_name in self._handlers:
            if event_name not in self._listener_tasks:
                task = asyncio.create_task(self._listen_event(event_name))
                self._listener_tasks[event_name] = task

        logger.debug(f"Broadcaster 事件总线开始监听，事件数: {len(self._handlers)}")

    async def close(self) -> None:
        """关闭事件总线。"""
        self._running = False

        # 取消所有监听任务
        for task in self._listener_tasks.values():
            task.cancel()
        self._listener_tasks.clear()

        # 关闭连接
        if self._broadcast:
            await self._broadcast.disconnect()
            self._broadcast = None

        self._handlers.clear()
        logger.debug("Broadcaster 事件总线已关闭")


__all__ = ["BroadcasterEventBus"]



"""便捷的链路追踪 API。

提供简洁的 span 创建方式，用于手动追踪自定义业务逻辑。

用法:
    # 上下文管理器方式（kind 可自定义，如 "llm"、"tool" 等）
    with span("llm.chat", kind="llm", model="gpt-4") as s:
        response = await call_openai()
        s.set_attribute("tokens", response.usage.total_tokens)
    
    # 装饰器方式
    @trace_span("tool.search", kind="tool")
    async def search(query: str):
        ...
"""

from __future__ import annotations

import asyncio
from collections.abc import Callable
from contextlib import contextmanager
from functools import wraps
from typing import TYPE_CHECKING, Any

# OTel 可选依赖
try:
    from opentelemetry import trace as otel_trace
    from opentelemetry.trace import SpanKind as OTelSpanKind
    from opentelemetry.trace import StatusCode
except ImportError:
    otel_trace = None  # type: ignore[assignment]
    OTelSpanKind = None  # type: ignore[assignment, misc]
    StatusCode = None  # type: ignore[assignment, misc]

if TYPE_CHECKING:
    from opentelemetry.trace import Span


class SpanKind:
    """OTel 标准 Span 类型常量。
    
    开发者也可传入任意字符串作为 kind（如 "llm"、"tool"），
    会作为属性记录，并默认映射为 INTERNAL。
    """
    
    INTERNAL = "internal"   # 内部操作
    CLIENT = "client"       # 外部调用（HTTP client、DB）
    SERVER = "server"       # 处理请求
    PRODUCER = "producer"   # 消息生产者
    CONSUMER = "consumer"   # 消息消费者


def _get_tracer():
    """获取 tracer 实例。"""
    if otel_trace is None:
        return None
    return otel_trace.get_tracer("aury")


def _to_otel_span_kind(kind: str):
    """转换为 OTel SpanKind。
    
    标准类型（internal/client/server/producer/consumer）映射到对应的 OTel SpanKind，
    其他自定义类型（如 llm/tool 等）默认映射为 INTERNAL。
    """
    if OTelSpanKind is None:
        return None
    
    mapping = {
        "internal": OTelSpanKind.INTERNAL,
        "client": OTelSpanKind.CLIENT,
        "server": OTelSpanKind.SERVER,
        "producer": OTelSpanKind.PRODUCER,
        "consumer": OTelSpanKind.CONSUMER,
    }
    return mapping.get(kind, OTelSpanKind.INTERNAL)


@contextmanager
def span(
    name: str,
    kind: str = "internal",
    **attributes: Any,
):
    """创建追踪 span（上下文管理器）。
    
    Args:
        name: span 名称，建议格式 "{category}.{operation}"
        kind: span 类型，可自由定义（如 "llm"、"tool"、"task" 等），
              标准类型（internal/client/server）会映射到 OTel SpanKind
        **attributes: 附加属性
    
    Yields:
        Span: OTel span 对象，可调用 set_attribute() 添加更多属性
    
    用法:
        # 追踪 LLM 调用
        with span("llm.chat", kind="llm", model="gpt-4") as s:
            response = await openai.chat.completions.create(...)
            s.set_attribute("llm.tokens", response.usage.total_tokens)
        
        # 追踪工具调用
        with span("tool.web_search", kind="tool", query=query):
            result = await search(query)
        
        # 追踪自定义业务
        with span("payment.process", order_id=order_id):
            await process_payment(order_id)
    """
    tracer = _get_tracer()
    
    if tracer is None:
        # OTel 未安装，使用空上下文
        yield _DummySpan()
        return
    
    otel_kind = _to_otel_span_kind(kind)
    
    with tracer.start_as_current_span(name, kind=otel_kind) as s:
        # 设置自定义类型标识（用于 AlertingSpanProcessor 识别）
        s.set_attribute("aury.span_kind", kind)
        
        # 设置用户传入的属性
        for key, value in attributes.items():
            if value is not None:
                s.set_attribute(key, _safe_attribute_value(value))
        
        yield s


def trace_span(
    name: str | None = None,
    kind: str = "internal",
    **attributes: Any,
) -> Callable:
    """追踪装饰器。
    
    Args:
        name: span 名称，默认使用函数名
        kind: span 类型，可自由定义
        **attributes: 附加属性
    
    用法:
        @trace_span(kind="llm", model="gpt-4")
        async def chat(prompt: str):
            return await openai.chat.completions.create(...)
        
        @trace_span("custom.operation")
        def sync_operation():
            ...
    """
    def decorator(func: Callable) -> Callable:
        span_name = name or f"{func.__module__}.{func.__qualname__}"
        
        @wraps(func)
        async def async_wrapper(*args, **kwargs):
            with span(span_name, kind=kind, **attributes):
                return await func(*args, **kwargs)
        
        @wraps(func)
        def sync_wrapper(*args, **kwargs):
            with span(span_name, kind=kind, **attributes):
                return func(*args, **kwargs)
        
        # 根据函数类型返回对应包装器
        if asyncio.iscoroutinefunction(func):
            return async_wrapper
        return sync_wrapper
    
    return decorator


def set_span_error(error: Exception, message: str | None = None) -> None:
    """在当前 span 上记录错误。
    
    Args:
        error: 异常对象
        message: 错误消息（可选）
    """
    if otel_trace is None or StatusCode is None:
        return
    
    current_span = otel_trace.get_current_span()
    if current_span and current_span.is_recording():
        current_span.set_status(StatusCode.ERROR, message or str(error))
        current_span.record_exception(error)


def set_span_attribute(key: str, value: Any) -> None:
    """在当前 span 上设置属性。
    
    Args:
        key: 属性名
        value: 属性值
    """
    if otel_trace is None:
        return
    
    current_span = otel_trace.get_current_span()
    if current_span and current_span.is_recording():
        current_span.set_attribute(key, _safe_attribute_value(value))


def _safe_attribute_value(value: Any) -> Any:
    """转换属性值为 OTel 支持的类型。"""
    if isinstance(value, str | int | float | bool):
        return value
    if isinstance(value, list | tuple):
        return [_safe_attribute_value(v) for v in value]
    return str(value)


class _DummySpan:
    """占位 span，用于 OTel 未安装时。"""
    
    def set_attribute(self, key: str, value: Any) -> None:
        pass
    
    def set_status(self, status, description: str | None = None) -> None:
        pass
    
    def record_exception(self, exception: Exception) -> None:
        pass
    
    def add_event(self, name: str, attributes: dict | None = None) -> None:
        pass


__all__ = [
    "SpanKind",
    "set_span_attribute",
    "set_span_error",
    "span",
    "trace_span",
]



"""默认中间件实现。

提供所有内置 HTTP 中间件的实现。
"""

from __future__ import annotations

from fastapi.middleware.cors import CORSMiddleware as FastAPICORSMiddleware
from starlette.middleware import Middleware as StarletteMiddleware

from aury.boot.application.app.base import FoundationApp, Middleware
from aury.boot.application.config import BaseConfig
from aury.boot.application.constants import MiddlewareName
from aury.boot.application.middleware.logging import (
    RequestLoggingMiddleware as StarletteRequestLoggingMiddleware,
)
from aury.boot.application.middleware.logging import (
    WebSocketLoggingMiddleware as StarletteWebSocketLoggingMiddleware,
)

__all__ = [
    "CORSMiddleware",
    "RequestLoggingMiddleware",
    "WebSocketLoggingMiddleware",
]


class RequestLoggingMiddleware(Middleware):
    """请求日志中间件。

    自动记录所有 HTTP 请求的详细信息，包括：
    - 请求方法、路径、查询参数
    - 客户端IP、User-Agent
    - 响应状态码、耗时
    - 链路追踪 ID（X-Trace-ID / X-Request-ID）
    - 请求上下文（user_id, tenant_id 等用户注册的字段）
    
    注意：用户的认证中间件应设置 order < 100，以便在日志记录前设置用户信息。
    """

    name = MiddlewareName.REQUEST_LOGGING
    enabled = True
    order = 100  # 用户中间件可使用 0-99 在此之前执行

    def build(self, config: BaseConfig) -> StarletteMiddleware:
        """构建请求日志中间件实例。"""
        return StarletteMiddleware(StarletteRequestLoggingMiddleware)


class CORSMiddleware(Middleware):
    """CORS 跨域处理中间件。

    处理跨域资源共享（CORS）请求，允许配置：
    - 允许的源（origins）
    - 允许的方法（methods）
    - 允许的头（headers）
    - 凭证支持（credentials）
    """

    name = MiddlewareName.CORS
    enabled = True
    order = 110  # 在日志中间件之后执行

    def can_enable(self, config: BaseConfig) -> bool:
        """仅当配置了 origins 时启用。"""
        return self.enabled and bool(config.cors.origins)

    def build(self, config: BaseConfig) -> StarletteMiddleware:
        """构建 CORS 中间件实例。"""
        return StarletteMiddleware(
            FastAPICORSMiddleware,
            allow_origins=config.cors.origins,
            allow_credentials=config.cors.allow_credentials,
            allow_methods=config.cors.allow_methods,
            allow_headers=config.cors.allow_headers,
        )


class WebSocketLoggingMiddleware(Middleware):
    """WebSocket 日志中间件。

    记录 WebSocket 连接生命周期：
    - 连接建立/断开
    - 消息收发统计
    - 异常断开
    - 链路追踪 ID
    """

    name = MiddlewareName.WEBSOCKET_LOGGING
    enabled = True
    order = 101  # 紧随 HTTP 日志中间件

    def build(self, config: BaseConfig) -> StarletteMiddleware:
        """构建 WebSocket 日志中间件实例。"""
        return StarletteMiddleware(
            StarletteWebSocketLoggingMiddleware,
            log_messages=config.log.websocket_log_messages,
        )


# 设置默认中间件
FoundationApp.middlewares = [
    RequestLoggingMiddleware,
    WebSocketLoggingMiddleware,
    CORSMiddleware,
]



"""OTel Logs 集成。

将 loguru 日志接入 OpenTelemetry，导出到 OTLP（Loki/Elasticsearch 等）。

用法:
    # 在 TelemetryComponent 中自动配置
    TELEMETRY__LOGS_ENDPOINT=http://loki:3100
    
    # 或手动配置:
    setup_otel_logging(endpoint="http://loki:3100")
"""

from __future__ import annotations

import logging

from aury.boot.common.logging import logger

# OTel Logs 可选依赖
try:
    from opentelemetry._logs import set_logger_provider
    from opentelemetry.exporter.otlp.proto.grpc._log_exporter import OTLPLogExporter
    from opentelemetry.sdk._logs import LoggerProvider, LoggingHandler
    from opentelemetry.sdk._logs.export import BatchLogRecordProcessor
except ImportError:
    OTLPLogExporter = None  # type: ignore[assignment, misc]


def setup_otel_logging(
    endpoint: str,
    headers: dict[str, str] | None = None,
) -> None:
    """配置 OTel 日志集成。
    
    Args:
        endpoint: OTLP 日志导出端点（如 http://loki:3100）
        headers: OTLP 请求头（可选）
    """
    if OTLPLogExporter is None:
        logger.debug("OTLP 日志导出器未安装")
        return
    
    try:
        # 创建 LoggerProvider
        logger_provider = LoggerProvider()
        set_logger_provider(logger_provider)
        
        # 添加 OTLP 导出器
        exporter = OTLPLogExporter(
            endpoint=endpoint,
            headers=headers or None,
        )
        logger_provider.add_log_record_processor(
            BatchLogRecordProcessor(exporter)
        )
        
        # 添加标准 logging handler
        logging_handler = LoggingHandler(
            level="DEBUG",
            logger_provider=logger_provider,
        )
        
        # 配置 loguru 转发到标准 logging
        logging.getLogger().addHandler(logging_handler)
        
        logger.info(f"Logs OTLP 导出器已配置: {endpoint}")
    except Exception as e:
        logger.warning(f"OTLP 日志导出配置失败: {e}")


__all__ = [
    "setup_otel_logging",
]



"""Profiling 模块。

提供持续性能分析和问题时刻状态快照功能：
- Pyroscope 集成：持续采样生成火焰图
- 事件循环阻塞检测：检测同步代码阻塞协程

使用方式：
    # 通过配置启用
    PROFILING__ENABLED=true
    PROFILING__PYROSCOPE_ENDPOINT=http://pyroscope:4040
    
    # 事件循环阻塞检测
    PROFILING__BLOCKING_DETECTOR_ENABLED=true
    PROFILING__BLOCKING_THRESHOLD_MS=100
"""

from __future__ import annotations

import asyncio
import gc
import sys
import threading
import time
import traceback
from collections import deque
from dataclasses import dataclass, field
from datetime import datetime
from typing import TYPE_CHECKING, Any

from aury.boot.common.logging import logger

# Pyroscope 可选依赖
try:
    import pyroscope
    PYROSCOPE_AVAILABLE = True
except ImportError:
    pyroscope = None  # type: ignore[assignment]
    PYROSCOPE_AVAILABLE = False

# psutil 可选依赖（用于进程资源监控）
try:
    import psutil
    PSUTIL_AVAILABLE = True
except ImportError:
    psutil = None  # type: ignore[assignment]
    PSUTIL_AVAILABLE = False

if TYPE_CHECKING:
    from collections.abc import Callable


# =============================================================================
# 配置
# =============================================================================


@dataclass
class ProfilingConfig:
    """Profiling 配置。"""
    
    # Pyroscope 配置
    enabled: bool = False
    pyroscope_endpoint: str | None = None
    pyroscope_auth_token: str | None = None
    service_name: str = "aury-service"
    environment: str = "development"
    
    # 事件循环阻塞检测配置
    blocking_detector_enabled: bool = False
    blocking_check_interval_ms: float = 100
    blocking_threshold_ms: float = 100
    blocking_severe_threshold_ms: float = 500
    blocking_alert_enabled: bool = True
    blocking_alert_cooldown_seconds: float = 60
    blocking_max_history: int = 50
    
    # 滑动窗口统计（秒）
    blocking_stats_window_seconds: float = 300  # 5分钟
    
    # 标签
    tags: dict[str, str] = field(default_factory=dict)


# =============================================================================
# Pyroscope 集成
# =============================================================================


class PyroscopeProfiler:
    """Pyroscope 持续 Profiler。
    
    集成 Grafana Pyroscope 实现持续性能分析和火焰图生成。
    """
    
    def __init__(self, config: ProfilingConfig) -> None:
        self._config = config
        self._initialized = False
    
    def start(self) -> bool:
        """启动 Pyroscope profiling。"""
        if self._initialized:
            return True
        
        if not PYROSCOPE_AVAILABLE:
            logger.warning("Pyroscope 未安装，跳过 profiling 初始化 (pip install pyroscope-io)")
            return False
        
        if not self._config.pyroscope_endpoint:
            logger.warning("Pyroscope endpoint 未配置，跳过初始化")
            return False
        
        try:
            pyroscope.configure(
                application_name=self._config.service_name,
                server_address=self._config.pyroscope_endpoint,
                auth_token=self._config.pyroscope_auth_token or "",
                tags=self._config.tags,
            )
            self._initialized = True
            logger.info(
                f"Pyroscope profiling 已启动 | "
                f"endpoint={self._config.pyroscope_endpoint} "
                f"service={self._config.service_name}"
            )
            return True
        except Exception as e:
            logger.error(f"Pyroscope 初始化失败: {e}")
            return False
    
    def stop(self) -> None:
        """停止 Pyroscope profiling。"""
        if not self._initialized:
            return
        
        try:
            if PYROSCOPE_AVAILABLE:
                pyroscope.shutdown()
            self._initialized = False
            logger.info("Pyroscope profiling 已停止")
        except Exception as e:
            logger.warning(f"Pyroscope 关闭失败: {e}")
    
    @property
    def is_running(self) -> bool:
        """是否正在运行。"""
        return self._initialized


# =============================================================================
# 事件循环阻塞检测
# =============================================================================


@dataclass
class BlockingEvent:
    """阻塞事件记录。"""
    
    timestamp: datetime
    blocked_ms: float
    main_thread_stack: list[dict[str, Any]]  # 最佳堆栈（用户代码优先）
    all_sampled_stacks: list[list[dict[str, Any]]] = field(default_factory=list)  # 所有采样堆栈
    process_stats: dict[str, Any] | None = None


class EventLoopBlockingDetector:
    """事件循环阻塞检测器。
    
    原理：后台线程定期向事件循环投递任务，如果任务执行延迟超过阈值，
    说明事件循环被同步代码阻塞。此时自动捕获主线程调用栈和进程状态。
    
    用于排查：
    - 同步 I/O 阻塞协程
    - CPU 密集型代码阻塞事件循环
    - 死锁或长时间锁等待
    """
    
    def __init__(self, config: ProfilingConfig) -> None:
        self._config = config
        self._running = False
        self._thread: threading.Thread | None = None
        self._loop: asyncio.AbstractEventLoop | None = None
        self._blocking_events: list[BlockingEvent] = []
        self._lock = threading.Lock()
        # 滑动窗口统计：记录时间戳 (timestamp, is_block)
        self._check_history: deque[tuple[float, bool]] = deque()
        self._last_alert_time: float = 0
    
    def start(self, loop: asyncio.AbstractEventLoop | None = None) -> None:
        """启动阻塞检测。"""
        if self._running:
            return
        
        try:
            self._loop = loop or asyncio.get_running_loop()
        except RuntimeError:
            logger.warning("无法获取事件循环，阻塞检测器未启动")
            return
        
        self._running = True
        self._thread = threading.Thread(
            target=self._monitor_loop,
            daemon=True,
            name="blocking-detector",
        )
        self._thread.start()
        logger.info(
            f"事件循环阻塞检测已启动 | "
            f"阈值={self._config.blocking_threshold_ms}ms "
            f"严重阈值={self._config.blocking_severe_threshold_ms}ms"
        )
    
    def stop(self) -> None:
        """停止阻塞检测。"""
        self._running = False
        if self._thread:
            self._thread.join(timeout=1.0)
            self._thread = None
        logger.info("事件循环阻塞检测已停止")
    
    def _monitor_loop(self) -> None:
        """后台监控循环。"""
        while self._running and self._loop:
            try:
                start_time = time.perf_counter()
                future = asyncio.run_coroutine_threadsafe(self._ping(), self._loop)
                
                # 在等待期间连续采样堆栈
                sampled_stacks: list[list[dict[str, Any]]] = []
                sample_interval = 0.01  # 10ms 采样一次
                
                try:
                    # 轮询等待，同时采样堆栈
                    timeout = self._config.blocking_threshold_ms * 10 / 1000
                    deadline = time.perf_counter() + timeout
                    
                    while time.perf_counter() < deadline:
                        try:
                            future.result(timeout=sample_interval)
                            break  # 成功返回
                        except TimeoutError:
                            # 还在等待，采样当前堆栈
                            elapsed = (time.perf_counter() - start_time) * 1000
                            if elapsed > self._config.blocking_threshold_ms * 0.5:  # 超过阈值50%开始采样
                                stack = self._capture_main_thread_stack()
                                if stack and (not sampled_stacks or stack != sampled_stacks[-1]):
                                    sampled_stacks.append(stack)
                    else:
                        # 超时
                        elapsed_ms = (time.perf_counter() - start_time) * 1000
                        self._record_blocking(elapsed_ms, sampled_stacks)
                        self._record_check(is_block=True)
                        time.sleep(self._config.blocking_check_interval_ms / 1000)
                        continue
                
                except Exception:
                    pass
                
                elapsed_ms = (time.perf_counter() - start_time) * 1000
                is_blocked = elapsed_ms > self._config.blocking_threshold_ms
                if is_blocked:
                    self._record_blocking(elapsed_ms, sampled_stacks)
                
                # 记录检查历史（滑动窗口）
                self._record_check(is_blocked)
            except Exception:
                pass  # 事件循环可能已关闭
            
            time.sleep(self._config.blocking_check_interval_ms / 1000)
    
    async def _ping(self) -> None:
        """空操作，用于测量事件循环响应时间。"""
        pass
    
    def _record_blocking(
        self,
        blocked_ms: float,
        sampled_stacks: list[list[dict[str, Any]]] | None = None,
    ) -> None:
        """记录阻塞事件。"""
        
        # 优先使用采样的堆栈（阻塞期间捕获的），否则捕获当前堆栈
        if sampled_stacks:
            # 取用户代码最多的堆栈作为主堆栈
            stack = self._merge_sampled_stacks(sampled_stacks)
            # 去重保留所有不同的堆栈
            unique_stacks = self._dedupe_stacks(sampled_stacks)
        else:
            stack = self._capture_main_thread_stack()
            unique_stacks = [stack] if stack else []
        
        # 获取进程状态
        process_stats = self._capture_process_stats()
        
        event = BlockingEvent(
            timestamp=datetime.now(),
            blocked_ms=round(blocked_ms, 2),
            main_thread_stack=stack,
            all_sampled_stacks=unique_stacks,
            process_stats=process_stats,
        )
        
        with self._lock:
            self._blocking_events.append(event)
            if len(self._blocking_events) > self._config.blocking_max_history:
                self._blocking_events.pop(0)
        
        # 输出日志
        self._log_blocking(event)
        
        # 发送告警
        if self._config.blocking_alert_enabled and self._loop:
            self._maybe_send_alert(event)
    
    def _capture_main_thread_stack(self) -> list[dict[str, Any]]:
        """捕获主线程调用栈。"""
        main_thread_id = threading.main_thread().ident
        if not main_thread_id or main_thread_id not in sys._current_frames():
            return []
        
        frame = sys._current_frames()[main_thread_id]
        stack = []
        
        for filename, lineno, name, line in traceback.extract_stack(frame):
            # 只跳过检测器自身和 frozen 内部代码
            if "<frozen" in filename or "monitoring/profiling" in filename:
                continue
            
            stack.append({
                "file": filename,
                "line": lineno,
                "function": name,
                "code": line,
            })
        
        return stack[-20:]  # 保留最近 20 帧
    
    def _is_user_code(self, filename: str) -> bool:
        """判断是否为用户代码（非标准库/非三方库）。"""
        if not filename:
            return False
        is_stdlib = any(p in filename for p in (
            "/lib/python", "/Lib/Python", "/opt/homebrew/Cellar/python",
            "/.pyenv/", "/Python.framework/"
        ))
        is_site_packages = "site-packages" in filename or "dist-packages" in filename
        return not is_stdlib and not is_site_packages
    
    def _score_stack(self, stack: list[dict[str, Any]]) -> int:
        """评分堆栈：用户代码帧越多分数越高。"""
        return sum(1 for f in stack if self._is_user_code(f.get("file", "")))
    
    def _stack_signature(self, stack: list[dict[str, Any]]) -> str:
        """生成堆栈签名用于去重。"""
        return "|".join(f"{f.get('file', '')}:{f.get('line', '')}" for f in stack[-5:])
    
    def _dedupe_stacks(
        self, stacks: list[list[dict[str, Any]]]
    ) -> list[list[dict[str, Any]]]:
        """去重堆栈，保留唯一的堆栈。"""
        seen: set[str] = set()
        unique: list[list[dict[str, Any]]] = []
        for stack in stacks:
            sig = self._stack_signature(stack)
            if sig not in seen:
                seen.add(sig)
                unique.append(stack)
        return unique
    
    def _merge_sampled_stacks(
        self, sampled_stacks: list[list[dict[str, Any]]]
    ) -> list[dict[str, Any]]:
        """合并多次采样的堆栈，返回用户代码最多的。"""
        if not sampled_stacks:
            return []
        return max(sampled_stacks, key=self._score_stack)
    
    def _capture_process_stats(self) -> dict[str, Any] | None:
        """捕获当前进程状态（含 GC 信息）。"""
        stats: dict[str, Any] = {}
        
        # GC 统计（始终可用）
        try:
            gc_stats = gc.get_stats()
            stats["gc"] = {
                "gen0_collections": gc_stats[0].get("collections", 0),
                "gen1_collections": gc_stats[1].get("collections", 0),
                "gen2_collections": gc_stats[2].get("collections", 0),
                "gen0_collected": gc_stats[0].get("collected", 0),
                "gen1_collected": gc_stats[1].get("collected", 0),
                "gen2_collected": gc_stats[2].get("collected", 0),
            }
            # 检查是否刚刚进行过 GC
            gc_count = gc.get_count()
            stats["gc"]["pending"] = gc_count  # (gen0, gen1, gen2) 待回收对象数
        except Exception:
            pass
        
        # 进程统计（需要 psutil）
        if PSUTIL_AVAILABLE:
            try:
                proc = psutil.Process()
                with proc.oneshot():
                    stats["cpu_percent"] = proc.cpu_percent()
                    stats["memory_rss_mb"] = round(proc.memory_info().rss / 1024**2, 2)
                    stats["num_threads"] = proc.num_threads()
                    stats["num_fds"] = proc.num_fds() if hasattr(proc, "num_fds") else None
            except Exception:
                pass
        
        return stats if stats else None
    
    def _format_stack(self, stack: list[dict[str, Any]], limit: int = 5, highlight_user: bool = True) -> str:
        """格式化调用栈为字符串。"""
        lines = []
        for frame in stack[-limit:]:
            if frame.get("code"):
                filename = frame['file']
                is_user = self._is_user_code(filename)
                # 用户代码加前缀标记
                prefix = "→ " if (highlight_user and is_user) else "  "
                lines.append(f"{prefix}{filename}:{frame['line']} in {frame['function']}")
                lines.append(f"    > {frame['code']}")
        return "\n".join(lines)
    
    def _record_check(self, is_block: bool) -> None:
        """记录一次检查到滑动窗口。"""
        now = time.time()
        with self._lock:
            self._check_history.append((now, is_block))
            # 清理过期数据
            cutoff = now - self._config.blocking_stats_window_seconds
            while self._check_history and self._check_history[0][0] < cutoff:
                self._check_history.popleft()
    
    def _get_window_stats(self) -> tuple[int, int]:
        """获取时间窗口内的统计。
        
        Returns:
            (total_checks, total_blocks)
        """
        now = time.time()
        cutoff = now - self._config.blocking_stats_window_seconds
        total_checks = 0
        total_blocks = 0
        
        with self._lock:
            for ts, is_block in self._check_history:
                if ts >= cutoff:
                    total_checks += 1
                    if is_block:
                        total_blocks += 1
        
        return total_checks, total_blocks
    
    def _log_blocking(self, event: BlockingEvent) -> None:
        """输出阻塞日志。"""
        is_severe = event.blocked_ms >= self._config.blocking_severe_threshold_ms
        log_fn = logger.error if is_severe else logger.warning
        
        # 获取时间窗口统计
        total_checks, total_blocks = self._get_window_stats()
        window_minutes = int(self._config.blocking_stats_window_seconds / 60)
        
        # 格式化进程状态
        stats_str = ""
        gc_str = ""
        if event.process_stats:
            s = event.process_stats
            stats_str = f" | CPU={s.get('cpu_percent', 'N/A')}% RSS={s.get('memory_rss_mb', 'N/A')}MB threads={s.get('num_threads', 'N/A')}"
            # GC 信息
            if s.get("gc"):
                gc_info = s["gc"]
                pending = gc_info.get("pending", (0, 0, 0))
                gc_str = f"\nGC: gen0={gc_info.get('gen0_collections', 0)} gen1={gc_info.get('gen1_collections', 0)} gen2={gc_info.get('gen2_collections', 0)} | pending={pending}"
        
        # 检查是否有用户代码
        has_user_code = self._score_stack(event.main_thread_stack) > 0
        
        # 构建堆栈信息
        stack_lines = []
        
        if has_user_code:
            # 有用户代码，显示主堆栈
            stack_lines.append("调用栈 (→ 标记用户代码):")
            stack_lines.append(self._format_stack(event.main_thread_stack, limit=8))
        else:
            # 没有用户代码，可能是框架内部阻塞
            stack_lines.append("调用栈 (无用户代码，可能是三方库/框架内部阻塞):")
            stack_lines.append(self._format_stack(event.main_thread_stack, limit=5, highlight_user=False))
            
            # 显示所有不同的采样堆栈
            if len(event.all_sampled_stacks) > 1:
                stack_lines.append(f"\n共采样到 {len(event.all_sampled_stacks)} 个不同堆栈:")
                for i, stack in enumerate(event.all_sampled_stacks[:3], 1):  # 最多显示3个
                    if stack != event.main_thread_stack:
                        stack_lines.append(f"--- 采样 #{i} ---")
                        stack_lines.append(self._format_stack(stack, limit=3, highlight_user=False))
        
        log_fn(
            f"事件循环阻塞{'（严重）' if is_severe else ''}: {event.blocked_ms:.0f}ms "
            f"(阈值={self._config.blocking_threshold_ms}ms, "
            f"近{window_minutes}分钟={total_blocks}次, "
            f"阻塞率={total_blocks / max(total_checks, 1) * 100:.2f}%){stats_str}{gc_str}\n"
            + "\n".join(stack_lines)
        )
    
    def _maybe_send_alert(self, event: BlockingEvent) -> None:
        """发送告警（带冷却）。"""
        now = time.time()
        if now - self._last_alert_time < self._config.blocking_alert_cooldown_seconds:
            return
        
        self._last_alert_time = now
        asyncio.run_coroutine_threadsafe(self._send_alert(event), self._loop)
    
    async def _send_alert(self, event: BlockingEvent) -> None:
        """发送告警。"""
        try:
            from aury.boot.infrastructure.monitoring.alerting import (
                AlertEventType,
                AlertSeverity,
                emit_alert,
            )
            
            is_severe = event.blocked_ms >= self._config.blocking_severe_threshold_ms
            severity = AlertSeverity.CRITICAL if is_severe else AlertSeverity.WARNING
            
            # 获取时间窗口统计
            total_checks, total_blocks = self._get_window_stats()
            window_minutes = int(self._config.blocking_stats_window_seconds / 60)
            
            await emit_alert(
                AlertEventType.CUSTOM,
                f"事件循环阻塞{'（严重）' if is_severe else ''}: {event.blocked_ms:.0f}ms",
                severity=severity,
                source="blocking_detector",
                blocked_ms=event.blocked_ms,
                threshold_ms=self._config.blocking_threshold_ms,
                window_minutes=window_minutes,
                total_blocks=total_blocks,
                block_rate=f"{total_blocks / max(total_checks, 1) * 100:.2f}%",
                stacktrace=self._format_stack(event.main_thread_stack),
                process_stats=event.process_stats,
            )
        except Exception as e:
            logger.debug(f"发送阻塞告警失败: {e}")
    
    def get_status(self) -> dict[str, Any]:
        """获取检测状态和历史。"""
        total_checks, total_blocks = self._get_window_stats()
        window_minutes = int(self._config.blocking_stats_window_seconds / 60)
        
        with self._lock:
            events = [
                {
                    "timestamp": e.timestamp.isoformat(),
                    "blocked_ms": e.blocked_ms,
                    "stack": e.main_thread_stack,
                    "process_stats": e.process_stats,
                }
                for e in self._blocking_events
            ]
        
        return {
            "running": self._running,
            "config": {
                "check_interval_ms": self._config.blocking_check_interval_ms,
                "threshold_ms": self._config.blocking_threshold_ms,
                "severe_threshold_ms": self._config.blocking_severe_threshold_ms,
                "alert_enabled": self._config.blocking_alert_enabled,
                "stats_window_seconds": self._config.blocking_stats_window_seconds,
            },
            "stats": {
                "window_minutes": window_minutes,
                "total_checks": total_checks,
                "total_blocks": total_blocks,
                "block_rate_percent": round(
                    total_blocks / max(total_checks, 1) * 100, 2
                ),
            },
            "recent_events": events,
        }
    
    def clear_history(self) -> None:
        """清空阻塞历史。"""
        with self._lock:
            self._blocking_events.clear()
            self._check_history.clear()
    
    @property
    def is_running(self) -> bool:
        """是否正在运行。"""
        return self._running


# =============================================================================
# 统一管理器
# =============================================================================


class ProfilingManager:
    """Profiling 统一管理器。
    
    管理 Pyroscope 和阻塞检测器的生命周期。
    """
    
    _instance: "ProfilingManager | None" = None
    
    def __init__(self) -> None:
        self._config: ProfilingConfig | None = None
        self._pyroscope: PyroscopeProfiler | None = None
        self._blocking_detector: EventLoopBlockingDetector | None = None
    
    @classmethod
    def get_instance(cls) -> "ProfilingManager":
        """获取单例实例。"""
        if cls._instance is None:
            cls._instance = cls()
        return cls._instance
    
    def configure(self, config: ProfilingConfig) -> None:
        """配置管理器。"""
        self._config = config
        self._pyroscope = PyroscopeProfiler(config)
        self._blocking_detector = EventLoopBlockingDetector(config)
    
    async def start(self) -> None:
        """启动所有 profiling 组件。"""
        if not self._config:
            logger.warning("ProfilingManager 未配置")
            return
        
        # 启动 Pyroscope
        if self._config.enabled and self._pyroscope:
            self._pyroscope.start()
        
        # 启动阻塞检测器
        if self._config.blocking_detector_enabled and self._blocking_detector:
            self._blocking_detector.start()
    
    async def stop(self) -> None:
        """停止所有 profiling 组件。"""
        if self._pyroscope:
            self._pyroscope.stop()
        
        if self._blocking_detector:
            self._blocking_detector.stop()
    
    @property
    def pyroscope(self) -> PyroscopeProfiler | None:
        """获取 Pyroscope profiler。"""
        return self._pyroscope
    
    @property
    def blocking_detector(self) -> EventLoopBlockingDetector | None:
        """获取阻塞检测器。"""
        return self._blocking_detector
    
    def get_status(self) -> dict[str, Any]:
        """获取所有组件状态。"""
        return {
            "pyroscope": {
                "available": PYROSCOPE_AVAILABLE,
                "running": self._pyroscope.is_running if self._pyroscope else False,
            },
            "blocking_detector": (
                self._blocking_detector.get_status()
                if self._blocking_detector
                else {"running": False}
            ),
        }


# 便捷访问
def get_profiling_manager() -> ProfilingManager:
    """获取 ProfilingManager 实例。"""
    return ProfilingManager.get_instance()


__all__ = [
    "BlockingEvent",
    "EventLoopBlockingDetector",
    "ProfilingConfig",
    "ProfilingManager",
    "PyroscopeProfiler",
    "get_profiling_manager",
    "PSUTIL_AVAILABLE",
    "PYROSCOPE_AVAILABLE",
]



"""应用框架基类。

提供 FoundationApp、Middleware 和 Component 基类。
"""

from __future__ import annotations

from abc import ABC, abstractmethod
from collections.abc import Callable
from contextlib import asynccontextmanager
import sys
from typing import Any, ClassVar

from fastapi import FastAPI, Request, status
from fastapi.exceptions import RequestValidationError
from fastapi.responses import JSONResponse
from starlette.exceptions import HTTPException
from starlette.middleware import Middleware as StarletteMiddleware

from aury.boot.application.config import BaseConfig
from aury.boot.application.errors import global_exception_handler
from aury.boot.application.interfaces.egress import SuccessResponse
from aury.boot.common.logging import logger, register_log_sink, setup_logging
from aury.boot.infrastructure.cache import CacheManager
from aury.boot.infrastructure.database import DatabaseManager


class Middleware(ABC):
    """HTTP 中间件基类。

    用于处理 HTTP 请求的拦截和处理，如 CORS、请求日志等。
    中间件在应用构造阶段同步注册，在 HTTP 请求处理时执行。

    生命周期：
    1. ``build()`` - 在应用构造阶段（lifespan 之前）同步调用。
       返回 Starlette Middleware 实例，由框架统一注册。

    设计原则：
    - 专注 HTTP 请求处理
    - 同步注册（必须在 lifespan 之前）
    - 按 order 排序执行

    使用示例:
        class MyMiddleware(Middleware):
            name = "my_middleware"
            enabled = True
            order = 10  # 数字小的先执行

            def build(self, config: BaseConfig) -> StarletteMiddleware:
                return StarletteMiddleware(SomeMiddlewareClass, **options)
    """

    name: str = "middleware"
    enabled: bool = True
    order: int = 0  # 执行顺序，数字小的先执行

    def can_enable(self, config: BaseConfig) -> bool:
        """是否可以启用此中间件。

        子类可以重写此方法以实现条件化启用。
        默认检查 enabled 属性。

        Args:
            config: 应用配置

        Returns:
            是否启用
        """
        return self.enabled

    @abstractmethod
    def build(self, config: BaseConfig) -> StarletteMiddleware:
        """构建中间件实例。

        返回 Starlette Middleware 实例，由框架在构造 FastAPI 时统一注册。

        Args:
            config: 应用配置

        Returns:
            Starlette Middleware 实例
        """
        pass


# TODO: 未来考虑合并 Middleware/Plugin/Component 为统一的 Extension 概念：
# class Extension:
#     def build_middleware(self, config) -> StarletteMiddleware | None  # 可选：app 创建前
#     def install(self, app, config) -> None                           # 可选：app 创建后
#     async def setup(self, app, config) -> None                       # 可选：lifespan 启动
#     async def teardown(self, app) -> None                            # 可选：lifespan 关闭
# 这样一个扩展可以同时实现多个 hook，适用于 OTEL、认证等复杂场景。


class Plugin(ABC):
    """应用插件基类。

    用于需要 app 实例的扩展，如 OpenTelemetry instrument、Admin Console 挂载等。
    插件在 app 创建后、lifespan 之前同步执行。

    生命周期：
    1. ``install()`` - 在 app 创建后同步调用，可以访问 app 实例。

    与 Middleware 的区别：
    - Middleware: 在 app 创建前构建，返回 Starlette Middleware
    - Plugin: 在 app 创建后执行，可以操作 app 实例

    与 Component 的区别：
    - Component: 在 lifespan 里异步执行，用于基础设施初始化
    - Plugin: 在 lifespan 前同步执行，用于 app 扩展

    使用示例:
        class TelemetryPlugin(Plugin):
            name = "telemetry"
            enabled = True

            def install(self, app: FoundationApp, config: BaseConfig) -> None:
                FastAPIInstrumentor.instrument_app(app)
    """

    name: str = "plugin"
    enabled: bool = True

    def can_enable(self, config: BaseConfig) -> bool:
        """是否可以启用此插件。

        子类可以重写此方法以实现条件化启用。

        Args:
            config: 应用配置

        Returns:
            是否启用
        """
        return self.enabled

    @abstractmethod
    def install(self, app: FoundationApp, config: BaseConfig) -> None:
        """安装插件（同步，在 app 创建后、lifespan 之前调用）。

        Args:
            app: 应用实例
            config: 应用配置
        """
        pass

    def uninstall(self, app: FoundationApp) -> None:
        """卸载插件（可选，在 lifespan 关闭时调用）。

        用于清理资源。默认不做任何操作。

        Args:
            app: 应用实例
        """
        pass


class Component(ABC):
    """基础设施组件基类。

    用于管理基础设施的生命周期，如数据库、缓存、任务队列等。
    组件在应用启动时异步初始化，在应用关闭时异步清理。

    生命周期：
    1. ``setup()`` - 在应用启动时（lifespan 开始阶段）异步调用。
    2. ``teardown()`` - 在应用关闭时（lifespan 结束阶段）异步调用。

    使用示例:
        class MyService(Component):
            name = "my_service"
            enabled = True
            depends_on = ["database", "cache"]

            async def setup(self, app: FoundationApp, config: BaseConfig):
                # 异步初始化逻辑
                pass

            async def teardown(self, app: FoundationApp):
                # 清理逻辑
                pass
    """

    name: str = "component"
    enabled: bool = True
    depends_on: ClassVar[list[str]] = []

    def can_enable(self, config: BaseConfig) -> bool:
        """是否可以启用此组件。

        子类可以重写此方法以实现条件化启用。
        默认检查 enabled 属性。

        Args:
            config: 应用配置

        Returns:
            是否启用
        """
        return self.enabled

    @abstractmethod
    async def setup(self, app: FoundationApp, config: BaseConfig) -> None:
        """组件启动时调用（异步，在 lifespan 开始阶段）。

        Args:
            app: 应用实例
            config: 应用配置
        """
        pass

    @abstractmethod
    async def teardown(self, app: FoundationApp) -> None:
        """组件关闭时调用（异步，在 lifespan 结束阶段）。

        用于清理资源。

        Args:
            app: 应用实例
        """
        pass


class FoundationApp(FastAPI):
    """优雅、抽象、可扩展的应用框架。

    设计特点：
    - 分离：中间件和组件分开管理，职责清晰
    - 可扩展：只需改类属性，即可添加/移除/替换功能
    - 不受限：无固定的初始化步骤，组件按依赖关系动态排序

    中间件（Middleware）：
        处理 HTTP 请求拦截，如 CORS、请求日志等。
        在应用构造阶段同步注册，按 order 排序执行。

    组件（Component）：
        管理基础设施生命周期，如数据库、缓存、任务队列等。
        在应用启动时异步初始化，按依赖关系拓扑排序。

    默认中间件（可覆盖）:
        - RequestLoggingMiddleware（请求日志）
        - CORSMiddleware（跨域处理）

    默认组件（可覆盖）:
        - DatabaseComponent（数据库）
        - CacheComponent（缓存）
        - TaskComponent（任务队列）
        - SchedulerComponent（调度器）
        - MigrationComponent（数据库迁移）

    使用示例:
        # 基础应用
        app = FoundationApp()

        # 自定义应用
        class MyApp(FoundationApp):
            middlewares = [
                RequestLoggingMiddleware,
                CORSMiddleware,
            ]
            components = [
                DatabaseComponent,
                CacheComponent,
                MyCustomComponent,
            ]
    """

    # 默认中间件列表（子类可以覆盖）
    middlewares: ClassVar[list[type[Middleware] | Middleware]] = []

    # 默认插件列表（子类可以覆盖）
    plugins: ClassVar[list[type[Plugin] | Plugin]] = []

    # 默认组件列表（子类可以覆盖）
    components: ClassVar[list[type[Component] | Component]] = []

    def __init__(
        self,
        config: BaseConfig | None = None,
        *,
        title: str = "Aury Service",
        version: str = "1.0.0",
        description: str | None = None,
        logger_levels: list[tuple[str, str]] | None = None,
        extra_components: list[type[Component] | Component] | None = None,
        **kwargs: Any,
    ) -> None:
        """初始化应用。

        Args:
            config: 应用配置（可选，默认从环境变量加载）
            title: 应用标题
            version: 应用版本
            description: 应用描述
            logger_levels: 需要设置特定级别的 logger 列表，格式: [("name", "LEVEL"), ...]
                例如: [("sse_starlette", "WARNING"), ("httpx", "INFO")]
            extra_components: 额外的组件列表，会追加到默认组件之后
            **kwargs: 传递给 FastAPI 的其他参数
        """
        # 加载配置
        if config is None:
            config = BaseConfig()
        self._config = config
        
        # 记录调用者模块（用于自动发现 schedules 等模块）
        frame = sys._getframe(1)
        self._caller_module = frame.f_globals.get("__name__", "__main__")

        # 初始化日志（必须在其他操作之前）
        setup_logging(
            log_level=config.log.level,
            log_dir=config.log.dir,
            service_type=config.service.service_type,
            rotation_time=config.log.rotation_time,
            retention_days=config.log.retention_days,
            enable_file_rotation=config.log.enable_file_rotation,
            enable_console=config.log.enable_console,
            logger_levels=logger_levels,
            enqueue=config.log.enqueue,
        )
        
        # 注册 access 日志（HTTP 请求日志）
        register_log_sink("access", filter_key="access")

        # 初始化中间件、插件和组件管理
        self._middlewares: dict[str, Middleware] = {}
        self._plugins: dict[str, Plugin] = {}
        self._components: dict[str, Component] = {}
        self._lifecycle_listeners: dict[str, list[Callable]] = {}

        # 保存额外组件
        self._extra_components = extra_components or []

        # 创建生命周期管理器
        @asynccontextmanager
        async def lifespan(app: FastAPI):
            """应用生命周期管理。"""
            # 启动
            await self._on_startup()
            yield
            # 关闭
            await self._on_shutdown()
            # 卸载插件
            self._uninstall_plugins()

        # 收集中间件、插件和组件实例并过滤
        self._collect_middlewares()
        self._collect_plugins()
        self._collect_components()

        # 构建中间件实例列表，传给 FastAPI
        middleware_instances = self._build_middlewares()

        # 调用父类构造函数
        super().__init__(
            title=title,
            version=version,
            description=description,
            lifespan=lifespan,
            middleware=middleware_instances,
            **kwargs,
        )

        # 安装插件（app 创建后、lifespan 之前）
        self._install_plugins()

        # 异常处理：显式注册以覆盖 FastAPI/Starlette 默认处理器，确保统一响应格式
        self.add_exception_handler(RequestValidationError, global_exception_handler)  # 422 参数校验
        self.add_exception_handler(HTTPException, global_exception_handler)  # 4xx/5xx HTTP 异常
        self.add_exception_handler(Exception, global_exception_handler)  # 其他未处理异常

        # 设置路由
        self.setup_routes()

    def _collect_middlewares(self) -> None:
        """收集并实例化所有中间件。

        这一步在 super().__init__() 之前执行，只做实例化和过滤，
        不调用任何需要 app 已完成构造的方法。
        """
        for item in self.middlewares:
            # 支持类或实例
            if isinstance(item, type):
                middleware = item()
            else:
                middleware = item

            if middleware.can_enable(self._config):
                self._middlewares[middleware.name] = middleware
                logger.debug(f"中间件已收集: {middleware.name}")

    def _collect_plugins(self) -> None:
        """收集并实例化所有插件。

        这一步在 super().__init__() 之前执行，只做实例化和过滤。
        """
        for item in self.plugins:
            # 支持类或实例
            if isinstance(item, type):
                plugin = item()
            else:
                plugin = item

            if plugin.can_enable(self._config):
                self._plugins[plugin.name] = plugin
                logger.debug(f"插件已收集: {plugin.name}")

    def _collect_components(self) -> None:
        """收集并实例化所有组件。

        这一步在 super().__init__() 之前执行，只做实例化和过滤，
        不调用任何需要 app 已完成构造的方法。
        """
        # 合并默认组件和额外组件
        all_components = list(self.components) + list(self._extra_components)

        for item in all_components:
            # 支持类或实例
            if isinstance(item, type):
                component = item()
            else:
                component = item

            if component.can_enable(self._config):
                self._components[component.name] = component
                logger.debug(f"组件已收集: {component.name}")

    def _install_plugins(self) -> None:
        """安装所有插件。

        在 super().__init__() 之后、lifespan 之前同步调用。
        此时 app 已创建，插件可以访问 app 实例。
        """
        for plugin in self._plugins.values():
            try:
                plugin.install(self, self._config)
                logger.info(f"插件已安装: {plugin.name}")
            except Exception as e:
                logger.warning(f"插件安装失败 ({plugin.name}): {e}")

    def _uninstall_plugins(self) -> None:
        """卸载所有插件。

        在 lifespan 关闭后同步调用。
        """
        # 反序卸载
        for plugin in reversed(list(self._plugins.values())):
            try:
                plugin.uninstall(self)
                logger.debug(f"插件已卸载: {plugin.name}")
            except Exception as e:
                logger.warning(f"插件卸载失败 ({plugin.name}): {e}")

    def _build_middlewares(self) -> list[StarletteMiddleware]:
        """构建所有中间件实例。

        在 super().__init__() 之前调用，返回中间件实例列表传给 FastAPI。
        按 order 升序排序，Starlette 会反向构建栈，最后的最先执行。

        Returns:
            Starlette Middleware 实例列表
        """
        # 按 order 降序排序（Starlette 反向构建，所以降序排列后最小的在最外层）
        sorted_middlewares = sorted(
            self._middlewares.values(),
            key=lambda m: m.order,
            reverse=True,
        )
        result = []
        for middleware in sorted_middlewares:
            instance = middleware.build(self._config)
            result.append(instance)
            logger.debug(f"中间件已构建: {middleware.name}")
        return result

    async def _on_startup(self) -> None:
        """启动所有组件。"""
        logger.info("应用启动中...")

        try:
            # 拓扑排序
            sorted_components = self._topological_sort(list(self._components.values()))

            # 启动组件
            for component in sorted_components:
                try:
                    await component.setup(self, self._config)
                    logger.info(f"组件启动成功: {component.name}")
                except Exception as e:
                    logger.error(f"组件启动失败 ({component.name}): {e}")
                    raise

            logger.info("应用启动完成")
            
            # 打印启动横幅和组件状态
            from aury.boot.application.app.startup import (
                collect_component_status,
                print_startup_banner,
            )
            components = collect_component_status()
            print_startup_banner(
                app_name=self.title,
                version=self.version,
                components=components,
            )

        except Exception as e:
            logger.error(f"应用启动异常: {e}")
            raise

    async def _on_shutdown(self) -> None:
        """关闭所有组件。"""
        logger.info("应用关闭中...")

        try:
            # 反序关闭
            sorted_components = self._topological_sort(
                list(self._components.values()),
                reverse=True,
            )

            for component in sorted_components:
                try:
                    await component.teardown(self)
                    logger.info(f"组件关闭成功: {component.name}")
                except Exception as e:
                    logger.warning(f"组件关闭失败 ({component.name}): {e}")

            logger.info("应用关闭完成")

        except Exception as e:
            logger.error(f"应用关闭异常: {e}")

    def _topological_sort(
        self,
        components: list[Component],
        reverse: bool = False,
    ) -> list[Component]:
        """拓扑排序组件。

        按照 depends_on 关系排序，确保依赖先启动。

        Args:
            components: 组件列表
            reverse: 是否反序（用于关闭）

        Returns:
            排序后的组件列表
        """
        # 构建图
        component_map = {comp.name: comp for comp in components}
        visited = set()
        result = []

        def visit(name: str, visiting: set) -> None:
            if name in visited:
                return

            if name in visiting:
                logger.warning(f"检测到循环依赖: {name}")
                return

            visiting.add(name)

            # 访问依赖
            if name in component_map:
                for dep in component_map[name].depends_on:
                    if dep in component_map:
                        visit(dep, visiting)

            visiting.remove(name)
            visited.add(name)

            if name in component_map:
                result.append(component_map[name])

        # 访问所有组件
        for comp in components:
            visit(comp.name, set())

        # 反序用于关闭
        if reverse:
            result.reverse()

        return result

    def setup_routes(self) -> None:
        """设置路由。

        子类可以重写此方法来自定义路由配置。
        """
        # 健康检查端点（默认启用）
        self.setup_health_check()

    def setup_health_check(self) -> None:
        """设置健康检查端点。

        使用 Aury 框架的默认健康检查逻辑。
        路径和启用状态可通过 BaseConfig.health_check 配置。

        子类可以重写此方法来自定义健康检查逻辑。
        """
        # 检查是否启用
        if not self._config.health_check.enabled:
            logger.debug("Aury 默认健康检查端点已禁用")
            return

        health_path = self._config.health_check.path

        @self.get(health_path, tags=["health"])
        async def health_check(request: Request) -> SuccessResponse:
            """Aury 框架默认健康检查端点。
            
            检查数据库、缓存等基础设施组件的健康状态。
            注意：此端点与服务自身的健康检查端点独立，可通过配置自定义路径。
            """
            health_status = {
                "status": "healthy",
                "checks": {},
            }

            # 检查数据库（如果已初始化）
            db_manager = DatabaseManager.get_instance()
            if db_manager._initialized:
                try:
                    await db_manager.health_check()
                    health_status["checks"]["database"] = "ok"
                except Exception as e:
                    health_status["status"] = "unhealthy"
                    health_status["checks"]["database"] = f"error: {e!s}"

            # 检查缓存（如果已初始化）
            cache_manager = CacheManager.get_instance()
            if cache_manager._backend:
                try:
                    await cache_manager.get("__health_check__", default=None)
                    health_status["checks"]["cache"] = "ok"
                except Exception as e:
                    health_status["checks"]["cache"] = f"error: {e!s}"

            # 返回状态码
            status_code = (
                status.HTTP_200_OK
                if health_status["status"] == "healthy"
                else status.HTTP_503_SERVICE_UNAVAILABLE
            )

            return JSONResponse(
                content=SuccessResponse(data=health_status).model_dump(mode="json"),
                status_code=status_code,
            )
        
        logger.info(f"Aury 默认健康检查端点已注册: {health_path}")

    @property
    def config(self) -> BaseConfig:
        """获取应用配置。"""
        return self._config


__all__ = [
    "Component",
    "Middleware",
]




"""日志配置和初始化。

提供 setup_logging 和 register_log_sink 功能。
"""

from __future__ import annotations

import logging
import os
from typing import Any

from loguru import logger

# =============================================================================
# TRACE Level 支持
# =============================================================================
# 标准 logging 没有 TRACE，需要手动添加
# TRACE (5) < DEBUG (10)，用于超细粒度调试（如每个 streaming chunk）
TRACE = 5
logging.addLevelName(TRACE, "TRACE")


def _add_trace_method() -> None:
    """为标准 logging.Logger 添加 trace() 方法。"""
    def trace(self: logging.Logger, msg: str, *args: Any, **kwargs: Any) -> None:
        if self.isEnabledFor(TRACE):
            self._log(TRACE, msg, args, **kwargs)
    
    logging.Logger.trace = trace  # type: ignore[attr-defined]

_add_trace_method()

from aury.boot.common.logging.context import (
    ServiceContext,
    _to_service_context,
    get_service_context,
    get_trace_id,
    set_service_context,
)
from aury.boot.common.logging.format import create_console_sink, format_message

# 全局日志配置状态
_log_config: dict[str, Any] = {
    "log_dir": "logs",
    "rotation": "00:00",
    "retention_days": 7,
    "file_format": "",
    "initialized": False,
}


def register_log_sink(
    name: str,
    *,
    filter_key: str | None = None,
    level: str = "INFO",
    sink_format: str | None = None,
) -> None:
    """注册自定义日志 sink。
    
    使用 logger.bind() 标记的日志会写入对应文件。
    
    Args:
        name: 日志文件名前缀（如 "access" -> access_2024-01-01.log）
        filter_key: 过滤键名，日志需要 logger.bind(key=True) 才会写入
        level: 日志级别
        sink_format: 自定义格式（默认使用简化格式）
    
    使用示例:
        # 注册 access 日志
        register_log_sink("access", filter_key="access")
        
        # 写入 access 日志
        logger.bind(access=True).info("GET /api/users 200 0.05s")
    """
    if not _log_config["initialized"]:
        raise RuntimeError("请先调用 setup_logging() 初始化日志系统")
    
    log_dir = _log_config["log_dir"]
    rotation = _log_config["rotation"]
    retention_days = _log_config["retention_days"]
    
    default_format = (
        "{time:YYYY-MM-DD HH:mm:ss} | "
        "{extra[trace_id]} | "
        "{message}"
    )
    
    # 创建 filter
    if filter_key:
        def sink_filter(record, key=filter_key):
            return record["extra"].get(key, False)
    else:
        sink_filter = None
    
    logger.add(
        os.path.join(log_dir, f"{name}_{{time:YYYY-MM-DD}}.log"),
        rotation=rotation,
        retention=f"{retention_days} days",
        level=level,
        format=sink_format or default_format,
        encoding="utf-8",
        enqueue=_log_config.get("enqueue", False),
        delay=True,
        filter=sink_filter,
    )
    
    logger.debug(f"注册日志 sink: {name} (filter_key={filter_key})")



class _InterceptHandler(logging.Handler):
    """将标准 logging 日志转发到 loguru 的处理器。"""

    def emit(self, record: logging.LogRecord) -> None:
        # 获取对应的 loguru 级别
        # loguru 原生支持 TRACE，标准 logging 的 TRACE(5) 会自动映射
        try:
            level = logger.level(record.levelname).name
        except ValueError:
            level = record.levelno

        # 查找调用者的帧深度
        # 跳过 logging 模块和本文件的所有帧
        frame = logging.currentframe()
        depth = 0
        while frame is not None:
            filename = frame.f_code.co_filename
            # 跳过 logging 模块、本文件、loguru 内部
            if "logging" not in filename and "loguru" not in filename:
                break
            frame = frame.f_back
            depth += 1

        logger.opt(depth=depth, exception=record.exc_info).log(
            level, record.getMessage()
        )


def _setup_global_intercept(logger_levels: list[tuple[str, str]] | None = None) -> None:
    """全局接管所有标准 logging，转发到 loguru。
    
    这样任何使用 logging.getLogger() 的库都会自动被接管。
    
    Args:
        logger_levels: 需要设置特定级别的 logger 列表，格式: [("name", "LEVEL"), ...]
            例如: [("sse_starlette", "WARNING"), ("httpx", "INFO")]
    """
    logging.root.handlers = [_InterceptHandler()]
    logging.root.setLevel(TRACE)  # 接收所有级别，包括 TRACE
    
    # 对指定的 logger 设置特定级别
    if logger_levels:
        for name, level in logger_levels:
            logging.getLogger(name).setLevel(level.upper())


def setup_logging(
    log_level: str = "INFO",
    log_dir: str | None = None,
    service_type: ServiceContext | str = ServiceContext.API,
    enable_file_rotation: bool = True,
    rotation_time: str = "00:00",
    retention_days: int = 7,
    rotation_size: str = "50 MB",
    enable_console: bool = True,
    logger_levels: list[tuple[str, str]] | None = None,
    enqueue: bool = True,
) -> None:
    """设置日志配置。

    日志文件按服务类型分离：
    - {service_type}_info_{date}.log  - INFO/WARNING/DEBUG 日志
    - {service_type}_error_{date}.log - ERROR/CRITICAL 日志
    
    轮转策略：
    - 文件名包含日期，每天自动创建新文件
    - 单文件超过大小限制时，会轮转产生 .1, .2 等后缀
    
    可通过 register_log_sink() 注册额外的日志文件（如 access.log）。

    Args:
        log_level: 日志级别（DEBUG/INFO/WARNING/ERROR/CRITICAL/TRACE）
        log_dir: 日志目录（默认：./logs）
        service_type: 服务类型（app/scheduler/worker）
        enable_file_rotation: 是否启用日志轮转
        rotation_time: 每日轮转时间（默认：00:00）
        retention_days: 日志保留天数（默认：7 天）
        rotation_size: 单文件大小上限（默认：50 MB）
        enable_console: 是否输出到控制台
        logger_levels: 需要设置特定级别的 logger 列表，格式: [("name", "LEVEL"), ...]
            例如: [("sse_starlette", "WARNING"), ("httpx", "INFO")]
        enqueue: 是否启用后台线程写入（默认 True）。
            启用后日志在单独线程写入文件，避免阻塞事件循环。
            asyncio 应用必须设为 True，否则文件 I/O 会阻塞事件循环。
    """
    log_level = log_level.upper()
    log_dir = log_dir or "logs"
    os.makedirs(log_dir, exist_ok=True)
    
    # 滚动策略：基于大小轮转（文件名已包含日期，每天自动新文件）
    rotation = rotation_size if enable_file_rotation else None

    # 标准化服务类型
    service_type_enum = _to_service_context(service_type)

    # 清理旧的 sink，避免重复日志（idempotent）
    logger.remove()

    # 保存全局配置（供 register_log_sink 使用）
    _log_config.update({
        "log_dir": log_dir,
        "rotation": rotation,
        "retention_days": retention_days,
        "enqueue": enqueue,
        "initialized": True,
    })

    # 设置默认服务上下文
    set_service_context(service_type_enum)

    # 配置 patcher，确保每条日志都有 service 和 trace_id
    logger.configure(patcher=lambda record: (
        record["extra"].update({
            "trace_id": get_trace_id(),
            # 记录字符串值，便于过滤器比较
            "service": get_service_context().value,
        })
    ))

    # 控制台输出（使用 Java 风格堆栈）
    if enable_console:
        logger.add(
            create_console_sink(),
            format="{message}",  # 简单格式，避免解析 <module> 等函数名
            level=log_level,
            colorize=False,  # 颜色在 sink 内处理
        )

    # 为 app 和 scheduler 分别创建日志文件（通过 ContextVar 区分）
    # API 模式下会同时运行嵌入式 scheduler，需要两个文件
    contexts_to_create: list[str] = [service_type_enum.value]
    # API 模式下也需要 scheduler 日志文件
    if service_type_enum is ServiceContext.API:
        contexts_to_create.append(ServiceContext.SCHEDULER.value)
    
    for ctx in contexts_to_create:
        # INFO 级别文件（使用 Java 风格堆栈）
        info_file = os.path.join(
            log_dir,
            f"{ctx}_info_{{time:YYYY-MM-DD}}.log" if enable_file_rotation else f"{ctx}_info.log"
        )
        logger.add(
            info_file,
            format=lambda record: format_message(record),
            rotation=rotation,
            retention=f"{retention_days} days",
            level=log_level,  # >= INFO 都写入（包含 WARNING/ERROR/CRITICAL）
            encoding="utf-8",
            enqueue=enqueue,
            filter=lambda record, c=ctx: (
                record["extra"].get("service") == c
                and not record["extra"].get("access", False)
            ),
        )

        # ERROR 级别文件（使用 Java 风格堆栈）
        error_file = os.path.join(
            log_dir,
            f"{ctx}_error_{{time:YYYY-MM-DD}}.log" if enable_file_rotation else f"{ctx}_error.log"
        )
        logger.add(
            error_file,
            format=lambda record: format_message(record),
            rotation=rotation,
            retention=f"{retention_days} days",
            level="ERROR",
            encoding="utf-8",
            enqueue=enqueue,
            filter=lambda record, c=ctx: record["extra"].get("service") == c,
        )

    # 全局拦截标准 logging 日志并转发到 loguru
    # 所有使用 logging.getLogger() 的库自动被接管
    _setup_global_intercept(logger_levels=logger_levels)

    logger.info


__all__ = [
    "TRACE",
    "register_log_sink",
    "setup_logging",
]



"""调度器命令 - 独立运行调度器。

使用示例：
    aury scheduler              # 运行调度器
    aury scheduler --app main:app  # 指定应用模块
"""

from __future__ import annotations

import asyncio
from pathlib import Path
import sys

from rich.console import Console
import typer

console = Console()

app = typer.Typer(
    name="scheduler",
    help="🕐 独立运行调度器",
    no_args_is_help=False,
)


def _detect_app_module() -> str:
    """自动检测应用模块路径。"""
    import os

    # 1. 环境变量
    if app_module := os.environ.get("APP_MODULE"):
        return app_module

    # 2. pyproject.toml 中的 [tool.aury] 配置
    pyproject_path = Path.cwd() / "pyproject.toml"
    if pyproject_path.exists():
        try:
            import tomllib
            with open(pyproject_path, "rb") as f:
                data = tomllib.load(f)
            if package := data.get("tool", {}).get("aury", {}).get("package"):
                return f"{package}.main:app"
        except Exception:
            pass

    # 3. 默认
    return "main:app"


@app.callback(invoke_without_command=True)
def run_scheduler(
    ctx: typer.Context,
    app_path: str | None = typer.Option(
        None,
        "--app",
        "-a",
        help="应用模块路径（默认自动检测，如 main:app）",
    ),
) -> None:
    """独立运行调度器进程。

    调度器会加载应用中注册的所有定时任务并执行。

    示例：
        aury scheduler                    # 自动检测应用
        aury scheduler --app main:app     # 指定应用模块
        aury scheduler -a myapp.main:app  # 指定包中的应用
    """
    if ctx.invoked_subcommand is not None:
        return

    # 确保当前目录在 Python 路径中
    cwd = str(Path.cwd())
    if cwd not in sys.path:
        sys.path.insert(0, cwd)

    app_module = app_path or _detect_app_module()
    console.print("[bold cyan]🕐 启动独立调度器[/bold cyan]")
    console.print(f"   应用: [green]{app_module}[/green]")

    try:
        # 导入应用
        module_path, app_name = app_module.rsplit(":", 1)
        module = __import__(module_path, fromlist=[app_name])
        application = getattr(module, app_name)

        # 设置日志（必须在其他操作之前）
        from aury.boot.common.logging import setup_logging
        setup_logging(
            log_level=getattr(application, "_config", None) and application._config.log.level or "INFO",
            service_type="scheduler",
        )

        # 获取调度器
        from aury.boot.infrastructure.scheduler import SchedulerManager
        scheduler = SchedulerManager.get_instance()

        console.print("[bold green]✅ 调度器启动成功[/bold green]")
        console.print("[dim]按 Ctrl+C 停止[/dim]")

        # 运行调度器
        async def _run():
            await scheduler.initialize()
            scheduler.start()
            try:
                # 保持运行
                while True:
                    await asyncio.sleep(1)
            except asyncio.CancelledError:
                pass
            finally:
                scheduler.shutdown()

        asyncio.run(_run())

    except KeyboardInterrupt:
        console.print("\n[yellow]调度器已停止[/yellow]")
    except ImportError as e:
        console.print(f"[red]❌ 无法导入应用: {e}[/red]")
        console.print("[dim]请确保应用模块路径正确，如 main:app[/dim]")
        raise typer.Exit(1)
    except Exception as e:
        console.print(f"[red]❌ 启动失败: {e}[/red]")
        raise typer.Exit(1)


__all__ = ["app"]



"""代码生成器命令。

生成符合 Aury 规范的代码文件：
- model: SQLAlchemy 模型
- repo: Repository 数据访问层
- service: Service 业务逻辑层
- api: FastAPI 路由
- schema: Pydantic 模型
- crud: 一键生成以上所有

支持两种字段定义模式：
1. 命令行参数（AI 友好）：
   aury generate model user email:str:unique age:int? status:str=active

2. 交互式（人类友好）：
   aury generate model user -i
"""

from __future__ import annotations

from dataclasses import dataclass, field
from pathlib import Path
import re
from typing import Annotated

from rich.console import Console
from rich.panel import Panel
from rich.prompt import Confirm, Prompt
from rich.table import Table
import typer

from .config import get_project_config

console = Console()

# 模板目录
GENERATE_TEMPLATES_DIR = Path(__file__).parent / "templates" / "generate"

# 创建代码生成器子应用
app = typer.Typer(
    name="generate",
    help="代码生成器 - 生成 model/repo/service/api/schema",
    no_args_is_help=True,
)


# ============================================================
# 字段解析
# ============================================================

# 类型映射：简写 -> (SQLAlchemy 类型, Pydantic 类型, 需要的导入)
TYPE_MAPPING: dict[str, tuple[str, str, list[str]]] = {
    # 字符串
    "str": ("String(255)", "str", ["String"]),
    "string": ("String(255)", "str", ["String"]),
    "text": ("Text", "str", ["Text"]),
    # 数字
    "int": ("Integer", "int", ["Integer"]),
    "integer": ("Integer", "int", ["Integer"]),
    "bigint": ("BigInteger", "int", ["BigInteger"]),
    "float": ("Float", "float", ["Float"]),
    "decimal": ("Numeric(10, 2)", "Decimal", ["Numeric"]),
    # 布尔
    "bool": ("Boolean", "bool", ["Boolean"]),
    "boolean": ("Boolean", "bool", ["Boolean"]),
    # 日期时间
    "datetime": ("DateTime", "datetime", ["DateTime"]),
    "date": ("Date", "date", ["Date"]),
    "time": ("Time", "time", ["Time"]),
    # JSON
    "json": ("JSON", "dict", ["JSON"]),
    "dict": ("JSON", "dict", ["JSON"]),
    # UUID
    "uuid": ("GUID", "str", []),  # 使用框架内置 GUID
}


@dataclass
class FieldDefinition:
    """字段定义。"""

    name: str
    type_name: str = "str"
    nullable: bool = False
    unique: bool = False
    index: bool = False
    default: str | None = None
    max_length: int | None = None  # 用于 str 类型
    comment: str | None = None

    @classmethod
    def parse(cls, spec: str) -> "FieldDefinition":
        """解析字段定义字符串。

        格式: name:type:modifiers
        修饰符:
        - ? 或 nullable: 可空
        - unique: 唯一
        - index: 索引
        - =value: 默认值
        - (length): 长度限制

        示例:
        - email:str:unique
        - age:int?
        - status:str=active
        - name:str(100)
        - bio:text?
        - price:decimal:index
        """
        parts = spec.split(":")
        name = parts[0]

        # 默认类型为 str
        type_name = "str"
        nullable = False
        unique = False
        index = False
        default = None
        max_length = None

        for part in parts[1:]:
            # 检查是否是类型定义
            type_match = re.match(r"^([a-z]+)(\((\d+)\))?$", part.lower())
            if type_match and type_match.group(1) in TYPE_MAPPING:
                type_name = type_match.group(1)
                if type_match.group(3):
                    max_length = int(type_match.group(3))
                continue

            # 检查修饰符
            part_lower = part.lower()
            if part_lower in ("?", "nullable"):
                nullable = True
            elif part_lower == "unique":
                unique = True
            elif part_lower == "index":
                index = True
            elif part.startswith("="):
                default = part[1:]
            elif part.endswith("?"):
                # 处理 age:int? 这种格式
                type_check = part[:-1].lower()
                if type_check in TYPE_MAPPING:
                    type_name = type_check
                    nullable = True

        # 处理名字后面直接跟 ? 的情况，如 "age?"
        if name.endswith("?"):
            name = name[:-1]
            nullable = True

        return cls(
            name=name,
            type_name=type_name,
            nullable=nullable,
            unique=unique,
            index=index,
            default=default,
            max_length=max_length,
        )


# 可用的模型基类
# id_type: "int" | "uuid" 用于生成正确的 Schema/Service/API 类型
MODEL_BASE_CLASSES = {
    "IDOnlyModel": {
        "desc": "纯 int 主键（无时间戳，适合关系表）",
        "features": ["id: int"],
        "id_type": "int",
        "has_timestamps": False,
    },
    "UUIDOnlyModel": {
        "desc": "纯 UUID 主键（无时间戳，适合关系表）",
        "features": ["id: UUID"],
        "id_type": "uuid",
        "has_timestamps": False,
    },
    "Model": {
        "desc": "标准模型（int主键 + 时间戳）",
        "features": ["id: int", "created_at", "updated_at"],
        "id_type": "int",
        "has_timestamps": True,
    },
    "AuditableStateModel": {
        "desc": "int 主键 + 软删除（推荐）",
        "features": ["id: int", "created_at", "updated_at", "deleted_at"],
        "id_type": "int",
        "has_timestamps": True,
    },
    "UUIDModel": {
        "desc": "UUID 主键模型",
        "features": ["id: UUID", "created_at", "updated_at"],
        "id_type": "uuid",
        "has_timestamps": True,
    },
    "UUIDAuditableStateModel": {
        "desc": "UUID 主键 + 软删除",
        "features": ["id: UUID", "created_at", "updated_at", "deleted_at"],
        "id_type": "uuid",
        "has_timestamps": True,
    },
    "VersionedModel": {
        "desc": "乐观锁模型（int主键 + version）",
        "features": ["id: int", "version"],
        "id_type": "int",
        "has_timestamps": False,
    },
    "VersionedTimestampedModel": {
        "desc": "乐观锁 + 时间戳",
        "features": ["id: int", "created_at", "updated_at", "version"],
        "id_type": "int",
        "has_timestamps": True,
    },
    "VersionedUUIDModel": {
        "desc": "UUID + 乐观锁 + 时间戳",
        "features": ["id: UUID", "created_at", "updated_at", "version"],
        "id_type": "uuid",
        "has_timestamps": True,
    },
    "FullFeaturedModel": {
        "desc": "int 主键 + 全功能",
        "features": ["id: int", "created_at", "updated_at", "deleted_at", "version"],
        "id_type": "int",
        "has_timestamps": True,
    },
    "FullFeaturedUUIDModel": {
        "desc": "UUID 主键 + 全功能",
        "features": ["id: UUID", "created_at", "updated_at", "deleted_at", "version"],
        "id_type": "uuid",
        "has_timestamps": True,
    },
}

# UUID 类型的基类名称
UUID_BASE_CLASSES = {k for k, v in MODEL_BASE_CLASSES.items() if v.get("id_type") == "uuid"}


@dataclass
class ModelDefinition:
    """模型定义。"""

    name: str
    fields: list[FieldDefinition] = field(default_factory=list)
    soft_delete: bool = True
    timestamps: bool = True
    base_class: str | None = None  # 用户指定的基类

    @property
    def id_type(self) -> str:
        """获取 id 类型：'int' 或 'uuid'。"""
        if self.base_class:
            return MODEL_BASE_CLASSES.get(self.base_class, {}).get("id_type", "int")
        # 默认使用 int 主键
        return "int"

    @property
    def id_py_type(self) -> str:
        """获取 Python/Pydantic 的 id 类型。"""
        return "int" if self.id_type == "int" else "UUID"

    @property
    def has_timestamps(self) -> bool:
        """是否有时间戳字段。"""
        if self.base_class:
            return MODEL_BASE_CLASSES.get(self.base_class, {}).get("has_timestamps", True)
        return self.timestamps

    @property
    def class_name(self) -> str:
        """PascalCase 类名。"""
        return _to_pascal_case(self.name)

    @property
    def file_name(self) -> str:
        """snake_case 文件名。"""
        return _to_snake_case(self.name)

    @property
    def table_name(self) -> str:
        """snake_case 复数表名。"""
        return _to_plural(self.file_name)

    @property
    def var_name(self) -> str:
        """变量名。"""
        return self.file_name

    @property
    def var_name_plural(self) -> str:
        """复数变量名。"""
        return _to_plural(self.file_name)


# ============================================================
# 工具函数
# ============================================================


def _to_snake_case(name: str) -> str:
    """转换为 snake_case。"""
    s1 = re.sub(r"(.)([A-Z][a-z]+)", r"\1_\2", name)
    return re.sub(r"([a-z0-9])([A-Z])", r"\1_\2", s1).lower()


def _to_pascal_case(name: str) -> str:
    """转换为 PascalCase。"""
    snake = _to_snake_case(name)
    return "".join(word.capitalize() for word in snake.split("_"))


def _to_plural(name: str) -> str:
    """简单的复数转换。"""
    if name.endswith("y"):
        return name[:-1] + "ies"
    if name.endswith(("s", "x", "ch", "sh")):
        return name + "es"
    return name + "s"


def _create_file(path: Path, content: str, force: bool = False) -> bool:
    """创建文件。"""
    if path.exists() and not force:
        return False
    path.parent.mkdir(parents=True, exist_ok=True)
    path.write_text(content, encoding="utf-8")
    return True


def _update_init_file(init_path: Path, import_line: str, export_name: str) -> None:
    """更新 __init__.py 文件。"""
    if not init_path.exists():
        init_path.write_text(
            f'{import_line}\n\n__all__ = ["{export_name}"]\n', encoding="utf-8"
        )
        return

    content = init_path.read_text(encoding="utf-8")

    # 检查是否已导入
    if import_line in content:
        return

    # 添加导入
    if "__all__" in content:
        content = content.replace("__all__", f"{import_line}\n\n__all__")
        if f'"{export_name}"' not in content:
            content = content.replace("__all__ = [", f'__all__ = [\n    "{export_name}",')
    else:
        content += f'\n{import_line}\n\n__all__ = ["{export_name}"]\n'

    init_path.write_text(content, encoding="utf-8")


# ============================================================
# 交互式字段收集
# ============================================================


def _collect_base_class_interactive() -> str:
    """交互式选择模型基类。"""
    console.print("\n[bold cyan]📚 选择模型基类[/bold cyan]\n")
    
    # 显示可用基类
    table = Table(title="可用模型基类", show_header=True, header_style="bold magenta")
    table.add_column("序号", style="dim", width=4)
    table.add_column("基类名", style="cyan")
    table.add_column("说明")
    table.add_column("自动继承的字段")
    
    base_names = list(MODEL_BASE_CLASSES.keys())
    for i, name in enumerate(base_names, 1):
        info = MODEL_BASE_CLASSES[name]
        # 推荐的加标记
        desc = info["desc"]
        if name == "AuditableStateModel":
            desc = f"[bold green]★ {desc}[/bold green]"
        table.add_row(str(i), name, desc, ", ".join(info["features"]))
    
    console.print(table)
    console.print()
    
    # 默认选择 AuditableStateModel（第 4 个）
    choice = Prompt.ask(
        "请选择基类序号",
        default="4",
        choices=[str(i) for i in range(1, len(base_names) + 1)],
    )
    
    selected = base_names[int(choice) - 1]
    console.print(f"  [green]✓ 已选择: {selected}[/green]\n")
    return selected


def _collect_fields_interactive() -> list[FieldDefinition]:
    """交互式收集字段定义。"""
    fields: list[FieldDefinition] = []

    console.print("\n[bold cyan]📝 添加字段[/bold cyan] (输入空名称结束)\n")

    # 显示类型帮助
    table = Table(title="支持的类型", show_header=True, header_style="bold magenta")
    table.add_column("类型", style="cyan")
    table.add_column("说明")
    table.add_row("str, string", "字符串 (默认)")
    table.add_row("text", "长文本")
    table.add_row("int, integer", "整数")
    table.add_row("bigint", "大整数")
    table.add_row("float", "浮点数")
    table.add_row("decimal", "精确小数")
    table.add_row("bool, boolean", "布尔值")
    table.add_row("datetime", "日期时间")
    table.add_row("date", "日期")
    table.add_row("json, dict", "JSON 对象")
    console.print(table)
    console.print()

    while True:
        name = Prompt.ask("[bold]字段名[/bold]", default="")
        if not name:
            break

        type_name = Prompt.ask(
            "  类型",
            default="str",
            choices=list(TYPE_MAPPING.keys()),
        )

        nullable = Confirm.ask("  可空?", default=False)
        unique = Confirm.ask("  唯一?", default=False)
        index = Confirm.ask("  索引?", default=False)
        default = Prompt.ask("  默认值 (留空无默认)", default="")

        max_length = None
        if type_name in ("str", "string"):
            length_str = Prompt.ask("  最大长度", default="255")
            max_length = int(length_str) if length_str.isdigit() else 255

        fields.append(
            FieldDefinition(
                name=name,
                type_name=type_name,
                nullable=nullable,
                unique=unique,
                index=index,
                default=default if default else None,
                max_length=max_length,
            )
        )

        console.print(f"  [green]✓ 已添加: {name}:{type_name}[/green]\n")

    return fields


# ============================================================
# 模板读取
# ============================================================


def _read_generate_template(name: str) -> str:
    """读取代码生成模板文件。"""
    template_path = GENERATE_TEMPLATES_DIR / name
    if not template_path.exists():
        raise FileNotFoundError(f"模板文件不存在: {name} (查找路径: {GENERATE_TEMPLATES_DIR})")
    return template_path.read_text(encoding="utf-8")


def _get_base_class_from_model_file(code_root: Path, model_name: str) -> str | None:
    """从已生成的模型文件中读取基类名称。
    
    Args:
        code_root: 代码根目录
        model_name: 模型名称（snake_case）
    
    Returns:
        基类名称，如果无法读取则返回 None
    """
    model_file = code_root / "models" / f"{model_name}.py"
    if not model_file.exists():
        return None
    
    try:
        content = model_file.read_text(encoding="utf-8")
        # 查找继承的基类，例如: class User(UUIDAuditableStateModel):
        pattern = r"class\s+\w+\s*\((\w+)\)"
        match = re.search(pattern, content)
        if match:
            base_class = match.group(1)
            # 验证是否是有效的基类
            if base_class in MODEL_BASE_CLASSES:
                return base_class
    except Exception:
        pass
    
    return None


# ============================================================
# 模板生成
# ============================================================


def _generate_model_content(model: ModelDefinition) -> str:
    """生成 Model 内容。"""
    # 收集需要的导入
    imports: set[str] = {"String"}  # 默认总是有 String
    for f in model.fields:
        type_info = TYPE_MAPPING.get(f.type_name, ("String(255)", "str", ["String"]))
        imports.update(type_info[2])

    imports_str = ", ".join(sorted(imports))

    # 生成字段定义
    field_lines = []
    for f in model.fields:
        type_info = TYPE_MAPPING.get(f.type_name, ("String(255)", "str", ["String"]))
        sa_type = type_info[0]
        py_type = type_info[1]

        # 处理字符串长度
        if f.type_name in ("str", "string") and f.max_length:
            sa_type = f"String({f.max_length})"

        # 构建 Mapped 类型
        if f.nullable:
            mapped_type = f"Mapped[{py_type} | None]"
        else:
            mapped_type = f"Mapped[{py_type}]"

        # 构建 mapped_column 参数
        col_args = [sa_type]
        if f.unique:
            col_args.append("unique=True")
        if f.index:
            col_args.append("index=True")
        if f.nullable:
            col_args.append("nullable=True")
        if f.default is not None:
            if f.type_name in ("str", "string", "text"):
                col_args.append(f'default="{f.default}"')
            elif f.type_name in ("bool", "boolean"):
                col_args.append(f"default={f.default.capitalize()}")
            else:
                col_args.append(f"default={f.default}")

        col_args_str = ", ".join(col_args)
        field_lines.append(f"    {f.name}: {mapped_type} = mapped_column({col_args_str})")

    fields_str = "\n".join(field_lines) if field_lines else "    # 添加字段"

    # 选择基类：优先使用用户指定的，否则根据选项推断
    if model.base_class:
        base_class = model.base_class
        base_info = MODEL_BASE_CLASSES.get(base_class, {})
        features = base_info.get("features", [])
        base_doc = f"继承 {base_class} 自动获得：\n    - " + "\n    - ".join(features) if features else f"继承 {base_class} 基类。"
    elif model.soft_delete and model.timestamps:
        base_class = "AuditableStateModel"
        base_doc = """继承 AuditableStateModel 自动获得：
    - id: int 自增主键
    - created_at: 创建时间
    - updated_at: 更新时间
    - deleted_at: 软删除时间戳"""
    elif model.timestamps:
        base_class = "Model"
        base_doc = """继承 Model 自动获得：
    - id: int 自增主键
    - created_at: 创建时间
    - updated_at: 更新时间"""
    else:
        base_class = "Model"
        base_doc = "继承 Model 基类。"

    template = _read_generate_template("model.py.tpl")
    return template.format(
        class_name=model.class_name,
        imports_str=imports_str,
        base_class=base_class,
        base_doc=base_doc,
        table_name=model.table_name,
        fields_str=fields_str,
    )


def _generate_schema_content(model: ModelDefinition) -> str:
    """生成 Schema 内容。"""
    # 基础字段
    base_fields = []
    update_fields = []

    for f in model.fields:
        type_info = TYPE_MAPPING.get(f.type_name, ("String(255)", "str", ["String"]))
        py_type = type_info[1]

        # Base 字段（Create 继承）
        if f.nullable:
            field_type = f"{py_type} | None"
            default = "None"
        elif f.default is not None:
            field_type = py_type
            if f.type_name in ("str", "string", "text"):
                default = f'"{f.default}"'
            elif f.type_name in ("bool", "boolean"):
                default = f.default.capitalize()
            else:
                default = f.default
        else:
            field_type = py_type
            default = "..."

        # 构建 Field 参数
        field_args = [default]
        if f.type_name in ("str", "string") and f.max_length:
            field_args.append(f"max_length={f.max_length}")
        field_args.append(f'description="{f.name}"')

        field_args_str = ", ".join(field_args)
        base_fields.append(f"    {f.name}: {field_type} = Field({field_args_str})")

        # Update 字段（全部可选）
        update_field_args = ["None"]
        if f.type_name in ("str", "string") and f.max_length:
            update_field_args.append(f"max_length={f.max_length}")
        update_field_args.append(f'description="{f.name}"')
        update_field_args_str = ", ".join(update_field_args)
        update_fields.append(
            f"    {f.name}: {py_type} | None = Field({update_field_args_str})"
        )

    base_fields_str = "\n".join(base_fields) if base_fields else "    pass"
    update_fields_str = "\n".join(update_fields) if update_fields else "    pass"

    # Response 字段（继承 Base，添加 id 和时间戳）
    id_type = model.id_py_type  # "int" 或 "UUID"
    
    # 根据是否有时间戳生成不同的 response 字段
    if model.has_timestamps:
        response_extra = f'''    id: {id_type} = Field(..., description="ID")
    created_at: datetime = Field(..., description="创建时间")
    updated_at: datetime = Field(..., description="更新时间")'''
    else:
        response_extra = f'    id: {id_type} = Field(..., description="ID")'

    # 导入语句
    imports = ["from datetime import datetime"] if model.has_timestamps else []
    if id_type == "UUID":
        imports.append("from uuid import UUID")
    imports_str = "\n".join(imports)
    if imports_str:
        imports_str += "\n"

    template = _read_generate_template("schema.py.tpl")
    return template.format(
        class_name=model.class_name,
        imports_str=imports_str,
        base_fields_str=base_fields_str,
        update_fields_str=update_fields_str,
        response_extra=response_extra,
    )


def _generate_repository_content(model: ModelDefinition, import_prefix: str = "") -> str:
    """生成 Repository 内容。

    Args:
        model: 模型定义
        import_prefix: import 前缀，如 "mypackage." 或 ""
    """
    # 查找唯一字段生成 get_by_xxx 方法
    get_by_methods = []
    for f in model.fields:
        if f.unique:
            type_info = TYPE_MAPPING.get(f.type_name, ("String(255)", "str", ["String"]))
            py_type = type_info[1]
            get_by_methods.append(f'''
    async def get_by_{f.name}(self, {f.name}: {py_type}) -> {model.class_name} | None:
        """按 {f.name} 获取。"""
        return await self.get_by({f.name}={f.name})''')

    methods_str = "\n".join(get_by_methods) if get_by_methods else ""

    template = _read_generate_template("repository.py.tpl")
    return template.format(
        class_name=model.class_name,
        import_prefix=import_prefix,
        file_name=model.file_name,
        methods_str=methods_str,
    )


def _generate_service_content(model: ModelDefinition, import_prefix: str = "") -> str:
    """生成 Service 内容。

    Args:
        model: 模型定义
        import_prefix: import 前缀，如 "mypackage." 或 ""
    """
    # 检查唯一字段，生成重复检测
    unique_checks = []
    for f in model.fields:
        if f.unique:
            unique_checks.append(
                f'''        # 检查 {f.name} 是否已存在
        existing = await self.repo.get_by_{f.name}(data.{f.name})
        if existing:
            raise AlreadyExistsError(f"{model.class_name} 已存在: {{data.{f.name}}}")
'''
            )

    unique_check_str = "\n".join(unique_checks) if unique_checks else ""
    
    # UUID 类型需要导入
    id_type = model.id_py_type
    uuid_import = "from uuid import UUID\n\n" if id_type == "UUID" else ""

    template = _read_generate_template("service.py.tpl")
    return template.format(
        class_name=model.class_name,
        uuid_import=uuid_import,
        import_prefix=import_prefix,
        file_name=model.file_name,
        id_py_type=model.id_py_type,
        unique_check_str=unique_check_str,
    )


def _generate_api_content(model: ModelDefinition, import_prefix: str = "") -> str:
    """生成 API 内容。

    Args:
        model: 模型定义
        import_prefix: import 前缀，如 "mypackage." 或 ""
    """
    id_type = model.id_py_type  # "int" 或 "UUID"
    
    # 导入语句
    imports = []
    if id_type == "UUID":
        imports.append("from uuid import UUID")
    imports_str = "\n".join(imports)
    if imports_str:
        imports_str += "\n"
    
    template = _read_generate_template("api.py.tpl")
    return template.format(
        class_name=model.class_name,
        uuid_import=imports_str,  # 模板中使用 uuid_import 占位符，但实际传入的是 imports_str（可能为空）
        import_prefix=import_prefix,
        file_name=model.file_name,
        var_name_plural=model.var_name_plural,
        var_name=model.var_name,
        id_type=id_type,
    )


# ============================================================
# 命令
# ============================================================


@app.command(name="model")
def generate_model(
    name: str = typer.Argument(..., help="模型名称（如 user, UserProfile）"),
    fields: Annotated[
        list[str] | None,
        typer.Argument(
            help="字段定义，格式: name:type:modifiers（如 email:str:unique age:int?）"
        ),
    ] = None,
    interactive: bool = typer.Option(
        False, "--interactive", "-i", help="交互式添加字段"
    ),
    base: str | None = typer.Option(
        None, "--base", "-b",
        help="模型基类（AuditableStateModel/Model/FullFeaturedModel 等）"
    ),
    force: bool = typer.Option(False, "--force", "-f", help="强制覆盖"),
    no_soft_delete: bool = typer.Option(False, "--no-soft-delete", help="禁用软删除"),
    no_timestamps: bool = typer.Option(False, "--no-timestamps", help="禁用时间戳"),
) -> None:
    """生成 SQLAlchemy 模型。

    支持两种模式：

    1. 命令行参数（AI 友好）:
        aury generate model user email:str:unique age:int? status:str=active

    2. 交互式（人类友好）:
        aury generate model user -i

    字段语法：
        name:type:modifiers

    支持的类型：
        str, text, int, bigint, float, decimal, bool, datetime, date, json

    修饰符：
        ? 或 nullable  - 可空
        unique         - 唯一约束
        index          - 索引
        =value         - 默认值
        (length)       - 字符串长度，如 str(100)

    可用基类：
        AuditableStateModel, Model, FullFeaturedModel,
        UUIDModel, UUIDAuditableStateModel, FullFeaturedUUIDModel,
        VersionedModel, VersionedTimestampedModel, VersionedUUIDModel

    示例：
        aury generate model user
        aury generate model user -b VersionedUUIDModel
        aury generate model user email:str:unique age:int?
        aury generate model article title:str(200) content:text status:str=draft
    """
    base_path = Path.cwd()

    # 读取项目配置
    config = get_project_config(base_path)
    code_root = config.get_package_dir(base_path)

    # 解析字段
    field_defs: list[FieldDefinition] = []
    selected_base_class: str | None = base

    if interactive:
        # 交互式模式：先选择基类，再添加字段
        selected_base_class = _collect_base_class_interactive()
        field_defs = _collect_fields_interactive()
    elif fields:
        for spec in fields:
            try:
                field_defs.append(FieldDefinition.parse(spec))
            except Exception as e:
                console.print(f"[red]❌ 解析字段失败: {spec} - {e}[/red]")
                raise typer.Exit(1) from e

    # 验证基类名称
    if selected_base_class and selected_base_class not in MODEL_BASE_CLASSES:
        console.print(f"[red]❌ 无效的基类: {selected_base_class}[/red]")
        console.print(f"[dim]可用基类: {', '.join(MODEL_BASE_CLASSES.keys())}[/dim]")
        raise typer.Exit(1)

    model = ModelDefinition(
        name=name,
        fields=field_defs,
        soft_delete=not no_soft_delete,
        timestamps=not no_timestamps,
        base_class=selected_base_class,
    )

    content = _generate_model_content(model)
    file_path = code_root / "models" / f"{model.file_name}.py"
    rel_path = file_path.relative_to(base_path)

    if _create_file(file_path, content, force):
        console.print(f"[green]✅ 创建模型: {rel_path}[/green]")
        _update_init_file(
            code_root / "models" / "__init__.py",
            f"from .{model.file_name} import {model.class_name}",
            model.class_name,
        )
    else:
        console.print(
            f"[yellow]⚠️  文件已存在: {rel_path}（使用 --force 覆盖）[/yellow]"
        )


@app.command(name="repo")
def generate_repo(
    name: str = typer.Argument(..., help="模型名称"),
    fields: Annotated[
        list[str] | None,
        typer.Argument(help="字段定义（用于生成 get_by_xxx 方法）"),
    ] = None,
    force: bool = typer.Option(False, "--force", "-f", help="强制覆盖"),
) -> None:
    """生成 Repository 数据访问层。

    示例：
        aury generate repo user
        aury generate repo user email:str:unique  # 生成 get_by_email 方法
    """
    base_path = Path.cwd()

    # 读取项目配置
    config = get_project_config(base_path)
    code_root = config.get_package_dir(base_path)
    import_prefix = config.get_import_prefix()

    field_defs = [FieldDefinition.parse(spec) for spec in (fields or [])]
    # 尝试从已生成的模型文件中读取基类信息
    base_class = _get_base_class_from_model_file(code_root, _to_snake_case(name))
    model = ModelDefinition(name=name, fields=field_defs, base_class=base_class)

    content = _generate_repository_content(model, import_prefix)
    file_path = code_root / "repositories" / f"{model.file_name}_repository.py"
    rel_path = file_path.relative_to(base_path)

    if _create_file(file_path, content, force):
        console.print(f"[green]✅ 创建仓储: {rel_path}[/green]")
        _update_init_file(
            code_root / "repositories" / "__init__.py",
            f"from .{model.file_name}_repository import {model.class_name}Repository",
            f"{model.class_name}Repository",
        )
    else:
        console.print("[yellow]⚠️  文件已存在（使用 --force 覆盖）[/yellow]")


@app.command(name="service")
def generate_service(
    name: str = typer.Argument(..., help="模型名称"),
    fields: Annotated[
        list[str] | None,
        typer.Argument(help="字段定义（用于生成重复检测）"),
    ] = None,
    force: bool = typer.Option(False, "--force", "-f", help="强制覆盖"),
) -> None:
    """生成 Service 业务逻辑层。

    示例：
        aury generate service user
        aury generate service user email:str:unique  # 创建时检查 email 重复
    """
    base_path = Path.cwd()

    # 读取项目配置
    config = get_project_config(base_path)
    code_root = config.get_package_dir(base_path)
    import_prefix = config.get_import_prefix()

    field_defs = [FieldDefinition.parse(spec) for spec in (fields or [])]
    # 尝试从已生成的模型文件中读取基类信息
    base_class = _get_base_class_from_model_file(code_root, _to_snake_case(name))
    model = ModelDefinition(name=name, fields=field_defs, base_class=base_class)

    content = _generate_service_content(model, import_prefix)
    file_path = code_root / "services" / f"{model.file_name}_service.py"
    rel_path = file_path.relative_to(base_path)

    if _create_file(file_path, content, force):
        console.print(f"[green]✅ 创建服务: {rel_path}[/green]")
        _update_init_file(
            code_root / "services" / "__init__.py",
            f"from .{model.file_name}_service import {model.class_name}Service",
            f"{model.class_name}Service",
        )
    else:
        console.print("[yellow]⚠️  文件已存在（使用 --force 覆盖）[/yellow]")


def _register_router_in_api_init(code_root: Path, model: ModelDefinition) -> bool:
    """自动在 api/__init__.py 中注册路由。
    
    Args:
        code_root: 代码根目录（包含 api/ 的目录）
        model: 模型定义
    
    Returns:
        是否成功注册
    """
    api_init_path = code_root / "api" / "__init__.py"
    if not api_init_path.exists():
        return False
    
    content = api_init_path.read_text(encoding="utf-8")
    
    # 检查是否已经注册
    import_line = f"from . import {model.file_name}"
    router_line = f"router.include_router({model.file_name}.router)"
    
    if import_line in content or f"{model.file_name}.router" in content:
        return False  # 已经注册
    
    # 查找插入位置：在 "# 注册子路由" 标记之后
    marker = "# 注册子路由"
    
    if marker not in content:
        return False
    
    try:
        # 在标记之后插入
        lines = content.split("\n")
        new_lines = []
        inserted = False
        
        for line in lines:
            new_lines.append(line)
            if marker in line and not inserted:
                new_lines.append(import_line)
                new_lines.append(router_line)
                inserted = True
        
        if inserted:
            api_init_path.write_text("\n".join(new_lines), encoding="utf-8")
            return True
    except Exception:
        pass  # 插入失败不影响主流程
    
    return False


@app.command(name="api")
def generate_api(
    name: str = typer.Argument(..., help="模型名称"),
    force: bool = typer.Option(False, "--force", "-f", help="强制覆盖"),
    no_register: bool = typer.Option(False, "--no-register", help="不自动注册到 api/__init__.py"),
) -> None:
    """生成 FastAPI 路由。

    示例：
        aury generate api user
        aury generate api user --no-register  # 不自动注册到 api/__init__.py
    """
    base_path = Path.cwd()

    # 读取项目配置
    config = get_project_config(base_path)
    code_root = config.get_package_dir(base_path)
    import_prefix = config.get_import_prefix()

    # 尝试从已生成的模型文件中读取基类信息
    base_class = _get_base_class_from_model_file(code_root, _to_snake_case(name))
    model = ModelDefinition(name=name, base_class=base_class)

    content = _generate_api_content(model, import_prefix)
    file_path = code_root / "api" / f"{model.file_name}.py"
    rel_path = file_path.relative_to(base_path)

    if _create_file(file_path, content, force):
        console.print(f"[green]✅ 创建 API: {rel_path}[/green]")
        
        # 自动注册到 api/__init__.py
        if not no_register:
            if _register_router_in_api_init(code_root, model):
                console.print("[green]✅ 已自动注册到 api/__init__.py[/green]")
            else:
                console.print("[dim]提示: 请在 api/__init__.py 中注册路由:[/dim]")
                console.print(f"[dim]  from . import {model.file_name}[/dim]")
                console.print(f"[dim]  router.include_router({model.file_name}.router)[/dim]")
    else:
        console.print("[yellow]⚠️  文件已存在（使用 --force 覆盖）[/yellow]")


@app.command(name="schema")
def generate_schema(
    name: str = typer.Argument(..., help="模型名称"),
    fields: Annotated[
        list[str] | None,
        typer.Argument(help="字段定义"),
    ] = None,
    interactive: bool = typer.Option(
        False, "--interactive", "-i", help="交互式添加字段"
    ),
    force: bool = typer.Option(False, "--force", "-f", help="强制覆盖"),
) -> None:
    """生成 Pydantic Schema。

    示例：
        aury generate schema user
        aury generate schema user email:str:unique age:int?
    """
    base_path = Path.cwd()

    # 读取项目配置
    config = get_project_config(base_path)
    code_root = config.get_package_dir(base_path)

    field_defs: list[FieldDefinition] = []
    if interactive:
        field_defs = _collect_fields_interactive()
    elif fields:
        field_defs = [FieldDefinition.parse(spec) for spec in fields]

    # 尝试从已生成的模型文件中读取基类信息
    base_class = _get_base_class_from_model_file(code_root, _to_snake_case(name))
    model = ModelDefinition(name=name, fields=field_defs, base_class=base_class)

    content = _generate_schema_content(model)
    file_path = code_root / "schemas" / f"{model.file_name}.py"
    rel_path = file_path.relative_to(base_path)

    if _create_file(file_path, content, force):
        console.print(f"[green]✅ 创建 Schema: {rel_path}[/green]")
        _update_init_file(
            code_root / "schemas" / "__init__.py",
            f"from .{model.file_name} import {model.class_name}Create, {model.class_name}Response, {model.class_name}Update",
            f"{model.class_name}Create",
        )
    else:
        console.print("[yellow]⚠️  文件已存在（使用 --force 覆盖）[/yellow]")


@app.command(name="crud")
def generate_crud(
    name: str = typer.Argument(..., help="模型名称"),
    fields: Annotated[
        list[str] | None,
        typer.Argument(help="字段定义"),
    ] = None,
    interactive: bool = typer.Option(
        False, "--interactive", "-i", help="交互式添加字段"
    ),
    base: str | None = typer.Option(
        None, "--base", "-b",
        help="模型基类（AuditableStateModel/Model/FullFeaturedModel 等）"
    ),
    force: bool = typer.Option(False, "--force", "-f", help="强制覆盖"),
    no_soft_delete: bool = typer.Option(False, "--no-soft-delete", help="禁用软删除"),
    no_timestamps: bool = typer.Option(False, "--no-timestamps", help="禁用时间戳"),
) -> None:
    """一键生成完整 CRUD（model + repo + service + api + schema）。

    支持两种模式：

    1. 命令行参数（AI 友好）:
        aury generate crud user email:str:unique age:int? status:str=active

    2. 交互式（人类友好）:
        aury generate crud user -i

    示例：
        aury generate crud user
        aury generate crud user --base AuditableStateModel  # int 主键 + 软删除（推荐）
        aury generate crud user --base Model  # int 主键 + 时间戳
        aury generate crud user email:str:unique age:int? --force
        aury generate crud article title:str(200) content:text status:str=draft
    """
    base_path = Path.cwd()
    
    # 读取项目配置
    config = get_project_config(base_path)
    code_root = config.get_package_dir(base_path)
    
    # 解析字段和基类
    field_defs: list[FieldDefinition] = []
    selected_base_class: str | None = base

    if interactive:
        # 交互式模式：先选择基类，再添加字段
        selected_base_class = _collect_base_class_interactive()
        field_defs = _collect_fields_interactive()
    elif fields:
        for spec in fields:
            try:
                field_defs.append(FieldDefinition.parse(spec))
            except Exception as e:
                console.print(f"[red]❌ 解析字段失败: {spec} - {e}[/red]")
                raise typer.Exit(1) from e

    # 验证基类名称
    if selected_base_class and selected_base_class not in MODEL_BASE_CLASSES:
        console.print(f"[red]❌ 无效的基类: {selected_base_class}[/red]")
        console.print(f"[dim]可用基类: {', '.join(MODEL_BASE_CLASSES.keys())}[/dim]")
        raise typer.Exit(1)

    model = ModelDefinition(
        name=name,
        fields=field_defs,
        soft_delete=not no_soft_delete,
        timestamps=not no_timestamps,
        base_class=selected_base_class,
    )

    console.print(
        Panel.fit(
            f"[bold cyan]⚡ 生成 CRUD: {model.class_name}[/bold cyan]",
            border_style="cyan",
        )
    )

    # 显示字段信息
    if model.fields:
        console.print("\n[bold]字段列表:[/bold]")
        for f in model.fields:
            modifiers = []
            if f.nullable:
                modifiers.append("nullable")
            if f.unique:
                modifiers.append("unique")
            if f.index:
                modifiers.append("index")
            if f.default:
                modifiers.append(f"default={f.default}")
            mod_str = f" [{', '.join(modifiers)}]" if modifiers else ""
            console.print(f"  • {f.name}: {f.type_name}{mod_str}")

    console.print()

    # 获取 import_prefix（项目配置已在前面读取）
    import_prefix = config.get_import_prefix()

    # 生成所有文件，直接使用已创建的 ModelDefinition 对象，确保基类信息一致
    # 1. 生成 Model
    model_content = _generate_model_content(model)
    model_file_path = code_root / "models" / f"{model.file_name}.py"
    model_rel_path = model_file_path.relative_to(base_path)
    if _create_file(model_file_path, model_content, force):
        console.print(f"[green]✅ 创建模型: {model_rel_path}[/green]")
        _update_init_file(
            code_root / "models" / "__init__.py",
            f"from .{model.file_name} import {model.class_name}",
            model.class_name,
        )
    else:
        console.print(f"[yellow]⚠️  文件已存在: {model_rel_path}（使用 --force 覆盖）[/yellow]")

    # 2. 生成 Repository
    repo_content = _generate_repository_content(model, import_prefix)
    repo_file_path = code_root / "repositories" / f"{model.file_name}_repository.py"
    repo_rel_path = repo_file_path.relative_to(base_path)
    if _create_file(repo_file_path, repo_content, force):
        console.print(f"[green]✅ 创建仓储: {repo_rel_path}[/green]")
        _update_init_file(
            code_root / "repositories" / "__init__.py",
            f"from .{model.file_name}_repository import {model.class_name}Repository",
            f"{model.class_name}Repository",
        )
    else:
        console.print(f"[yellow]⚠️  文件已存在: {repo_rel_path}（使用 --force 覆盖）[/yellow]")

    # 3. 生成 Service
    service_content = _generate_service_content(model, import_prefix)
    service_file_path = code_root / "services" / f"{model.file_name}_service.py"
    service_rel_path = service_file_path.relative_to(base_path)
    if _create_file(service_file_path, service_content, force):
        console.print(f"[green]✅ 创建服务: {service_rel_path}[/green]")
        _update_init_file(
            code_root / "services" / "__init__.py",
            f"from .{model.file_name}_service import {model.class_name}Service",
            f"{model.class_name}Service",
        )
    else:
        console.print(f"[yellow]⚠️  文件已存在: {service_rel_path}（使用 --force 覆盖）[/yellow]")

    # 4. 生成 Schema
    schema_content = _generate_schema_content(model)
    schema_file_path = code_root / "schemas" / f"{model.file_name}.py"
    schema_rel_path = schema_file_path.relative_to(base_path)
    if _create_file(schema_file_path, schema_content, force):
        console.print(f"[green]✅ 创建 Schema: {schema_rel_path}[/green]")
        _update_init_file(
            code_root / "schemas" / "__init__.py",
            f"from .{model.file_name} import {model.class_name}Create, {model.class_name}Response, {model.class_name}Update",
            f"{model.class_name}Create",
        )
    else:
        console.print(f"[yellow]⚠️  文件已存在: {schema_rel_path}（使用 --force 覆盖）[/yellow]")

    # 5. 生成 API
    api_content = _generate_api_content(model, import_prefix)
    api_file_path = code_root / "api" / f"{model.file_name}.py"
    api_rel_path = api_file_path.relative_to(base_path)
    if _create_file(api_file_path, api_content, force):
        console.print(f"[green]✅ 创建 API: {api_rel_path}[/green]")
        # 自动注册到 api/__init__.py
        if _register_router_in_api_init(code_root, model):
            console.print("[green]✅ 已自动注册到 api/__init__.py[/green]")
        else:
            console.print("[dim]提示: 请在 api/__init__.py 中注册路由:[/dim]")
            console.print(f"[dim]  from . import {model.file_name}[/dim]")
            console.print(f"[dim]  router.include_router({model.file_name}.router)[/dim]")
    else:
        console.print(f"[yellow]⚠️  文件已存在: {api_rel_path}（使用 --force 覆盖）[/yellow]")

    console.print()
    console.print("[bold green]✨ CRUD 生成完成！[/bold green]")
    console.print()
    console.print("[bold]下一步：[/bold]")
    console.print("  1. 生成数据库迁移：")
    console.print(f'     [cyan]aury migrate make -m "add {model.file_name} table"[/cyan]')
    console.print("  2. 执行迁移：")
    console.print("     [cyan]aury migrate up[/cyan]")


__all__ = ["app"]



"""第三方接口适配器配置模型。

提供统一的配置结构，支持：
- 多模式切换（real/sandbox/mock/disabled）
- 按方法级别的模式覆盖
- 通用连接参数（URL、认证、超时等）
- 挡板（Mock）策略配置

模式说明:
- real: 调用真实第三方接口（生产环境）
- sandbox: 调用第三方提供的沙箱环境
- mock: 使用本地挡板实现，不发出真实请求（测试/开发环境）
- disabled: 禁用该接口，调用时抛出 AdapterDisabledError
"""

from __future__ import annotations

from typing import Any, Literal

from pydantic import BaseModel, Field
from pydantic_settings import BaseSettings, SettingsConfigDict

# 适配器模式类型
# - real: 真实调用第三方接口
# - sandbox: 调用第三方沙箱环境
# - mock: 使用本地挡板实现
# - disabled: 禁用，调用时抛出异常
AdapterMode = Literal["real", "sandbox", "mock", "disabled"]

# 挡板（Mock）策略类型
# - success: 返回成功响应
# - failure: 返回失败响应
# - echo: 回显请求参数
# - noop: 空操作，返回空结果
# - custom: 自定义，使用 @method.mock 定义的处理函数
MockStrategy = Literal["success", "failure", "echo", "noop", "custom"]


class AdapterSettings(BaseModel):
    """第三方接口适配器配置。
    
    通用的第三方服务配置模型，支持多模式切换和按方法覆盖。
    
    配置项分类:
    1. 基础配置: enabled, mode
    2. 连接配置: base_url, sandbox_url, api_key, api_secret, timeout, retry_times
    3. 模式覆盖: method_modes - 按方法名覆盖全局模式
    4. 挡板配置: mock_strategy, mock_default_response, mock_delay, mock_method_responses
    
    环境变量配置示例:
        # 基础配置
        THIRD_PARTY_PAYMENT_ENABLED=true
        THIRD_PARTY_PAYMENT_MODE=mock          # 测试环境用挡板
        
        # 连接配置
        THIRD_PARTY_PAYMENT_BASE_URL=https://api.payment.com
        THIRD_PARTY_PAYMENT_SANDBOX_URL=https://sandbox.payment.com
        THIRD_PARTY_PAYMENT_API_KEY=sk_live_xxx
        THIRD_PARTY_PAYMENT_TIMEOUT=30
        
        # 方法级模式覆盖（JSON 格式）
        # query 走真实接口，refund 禁用
        THIRD_PARTY_PAYMENT_METHOD_MODES={"query": "real", "refund": "disabled"}
        
        # 挡板配置
        THIRD_PARTY_PAYMENT_MOCK_DELAY=0.1     # 模拟网络延迟 100ms
    """
    
    # ========== 基础配置 ==========
    enabled: bool = Field(
        default=True,
        description="是否启用此集成",
    )
    mode: AdapterMode = Field(
        default="real",
        description="当前模式：real(生产)/sandbox(沙箱)/mock(挡板)/disabled(禁用)",
    )
    
    # ========== 连接配置 ==========
    base_url: str | None = Field(
        default=None,
        description="服务基础 URL（生产环境）",
    )
    sandbox_url: str | None = Field(
        default=None,
        description="沙箱环境 URL（sandbox 模式时优先使用）",
    )
    api_key: str | None = Field(
        default=None,
        description="API 密钥（如 Authorization header）",
    )
    api_secret: str | None = Field(
        default=None,
        description="API 密钥（用于签名等）",
    )
    timeout: int = Field(
        default=30,
        description="请求超时（秒）",
    )
    retry_times: int = Field(
        default=3,
        description="重试次数",
    )
    
    # ========== 模式覆盖 ==========
    method_modes: dict[str, AdapterMode] = Field(
        default_factory=dict,
        description="按方法覆盖模式，如 {'refund': 'disabled', 'query': 'real'}",
    )
    
    # ========== Mock 配置 ==========
    mock_strategy: MockStrategy = Field(
        default="success",
        description="默认 mock 策略：success(成功)/failure(失败)/echo(回显)/noop(空操作)/custom(自定义)",
    )
    mock_default_response: dict[str, Any] | None = Field(
        default=None,
        description="默认 mock 响应（当没有自定义 mock handler 时使用）",
    )
    mock_delay: float = Field(
        default=0.0,
        description="mock 模拟延迟（秒），用于模拟网络延迟",
    )
    mock_method_responses: dict[str, dict[str, Any]] = Field(
        default_factory=dict,
        description="按方法的 mock 响应，如 {'create': {'success': True, 'id': 'mock-123'}}",
    )
    
    # ========== 扩展配置 ==========
    extra: dict[str, Any] = Field(
        default_factory=dict,
        description="额外配置（供子类或特定第三方使用）",
    )
    
    def get_effective_url(self) -> str | None:
        """获取当前模式下的有效 URL。
        
        sandbox 模式优先使用 sandbox_url，否则使用 base_url。
        """
        if self.mode == "sandbox" and self.sandbox_url:
            return self.sandbox_url
        return self.base_url
    
    def get_method_mode(self, method_name: str) -> AdapterMode:
        """获取指定方法的有效模式。
        
        优先使用 method_modes 中的配置，否则使用全局 mode。
        
        Args:
            method_name: 方法名称
            
        Returns:
            AdapterMode: 有效模式
        """
        return self.method_modes.get(method_name, self.mode)
    
    def is_method_enabled(self, method_name: str) -> bool:
        """检查指定方法是否启用。
        
        Args:
            method_name: 方法名称
            
        Returns:
            bool: 是否启用
        """
        if not self.enabled:
            return False
        return self.get_method_mode(method_name) != "disabled"
    
    def get_mock_response(self, method_name: str) -> dict[str, Any] | None:
        """获取指定方法的 mock 响应。
        
        优先使用 mock_method_responses，否则使用 mock_default_response。
        
        Args:
            method_name: 方法名称
            
        Returns:
            dict | None: mock 响应
        """
        return self.mock_method_responses.get(method_name, self.mock_default_response)


class ThirdPartySettings(BaseSettings):
    """第三方服务配置聚合。
    
    将所有第三方服务的配置聚合到一个类中，便于统一管理。
    项目可以继承此类添加更多第三方服务。
    
    环境变量前缀: THIRD_PARTY_
    
    使用示例:
        class MyThirdPartySettings(ThirdPartySettings):
            wechat: AdapterSettings = Field(default_factory=AdapterSettings)
            alipay: AdapterSettings = Field(default_factory=AdapterSettings)
        
        # 环境变量
        THIRD_PARTY_WECHAT_MODE=sandbox
        THIRD_PARTY_ALIPAY_MODE=mock
    """
    
    # 示例：常见第三方服务（项目可以继承并添加）
    # payment: AdapterSettings = Field(default_factory=AdapterSettings)
    # sms: AdapterSettings = Field(default_factory=AdapterSettings)
    # oss: AdapterSettings = Field(default_factory=AdapterSettings)
    # email: AdapterSettings = Field(default_factory=AdapterSettings)
    
    model_config = SettingsConfigDict(
        env_prefix="THIRD_PARTY_",
        env_nested_delimiter="__",
        case_sensitive=False,
        extra="ignore",
    )
    
    def set_all_mode(self, mode: AdapterMode) -> None:
        """将所有第三方服务设置为指定模式。
        
        便捷方法，用于测试环境一键切换所有服务到 mock 模式。
        
        Args:
            mode: 目标模式
        """
        for field_name in self.model_fields:
            field_value = getattr(self, field_name, None)
            if isinstance(field_value, AdapterSettings):
                field_value.mode = mode
    
    def disable_all(self) -> None:
        """禁用所有第三方服务。"""
        self.set_all_mode("disabled")
    
    def mock_all(self) -> None:
        """将所有第三方服务设置为 mock 模式。"""
        self.set_all_mode("mock")


__all__ = [
    "AdapterMode",
    "AdapterSettings",
    "MockStrategy",
    "ThirdPartySettings",
]



"""常用组合模型。

提供预定义的模型组合，方便直接使用。

**重要提示**：不要直接继承 Base 类用于 Repository，必须使用下列预定义模型或自己组合 Mixin。
直接使用 Base 会导致缺少 id 等必要字段，Repository 会报错。

可用模型：
- Model: 标准模型（int主键 + 时间戳）
- AuditableStateModel: 标准模型 + 软删除
- UUIDModel: UUID 主键模型（UUID 主键 + 时间戳）
- UUIDAuditableStateModel: UUID 主键模型 + 软删除
- VersionedModel: 乐观锁模型（int主键 + 乐观锁）
- VersionedTimestampedModel: 乐观锁 + 时间戳
- VersionedUUIDModel: UUID 主键 + 乐观锁 + 时间戳
- FullFeaturedModel: 完整功能（int主键 + 时间戳 + 软删除 + 乐观锁）
- FullFeaturedUUIDModel: 完整功能 UUID 版本
"""

from __future__ import annotations

from .base import Base
from .mixins import (
    AuditableStateMixin,
    IDMixin,
    TimestampMixin,
    UUIDMixin,
    VersionMixin,
)


class IDOnlyModel(IDMixin, Base):
    """纯 int 主键模型（无时间戳，适合关系表）"""

    __abstract__ = True


class UUIDOnlyModel(UUIDMixin, Base):
    """纯 UUID 主键模型（无时间戳，适合关系表）"""

    __abstract__ = True


class Model(IDMixin, TimestampMixin, Base):
    """【常用】标准整数主键模型"""

    __abstract__ = True


class AuditableStateModel(IDMixin, TimestampMixin, AuditableStateMixin, Base):
    """【常用】带可审计状态的标准模型（整数主键 + 时间戳 + 软删除）"""

    __abstract__ = True


class UUIDModel(UUIDMixin, TimestampMixin, Base):
    """【常用】UUID 主键模型"""

    __abstract__ = True


class UUIDAuditableStateModel(UUIDMixin, TimestampMixin, AuditableStateMixin, Base):
    """【常用】带可审计状态的 UUID 主键模型（UUID 主键 + 时间戳 + 软删除）"""

    __abstract__ = True


class VersionedModel(IDMixin, VersionMixin, Base):
    """整数主键 + 乐观锁"""

    __abstract__ = True


class VersionedTimestampedModel(IDMixin, TimestampMixin, VersionMixin, Base):
    """整数主键 + 时间戳 + 乐观锁"""

    __abstract__ = True


class VersionedUUIDModel(UUIDMixin, TimestampMixin, VersionMixin, Base):
    """UUID 主键 + 时间戳 + 乐观锁"""

    __abstract__ = True


class FullFeaturedModel(IDMixin, TimestampMixin, AuditableStateMixin, VersionMixin, Base):
    """完整功能：整数主键 + 时间戳 + 可审计状态 + 乐观锁"""

    __abstract__ = True


class FullFeaturedUUIDModel(UUIDMixin, TimestampMixin, AuditableStateMixin, VersionMixin, Base):
    """完整功能：UUID 主键 + 时间戳 + 可审计状态 + 乐观锁"""

    __abstract__ = True




"""缓存后端实现（兼容层）。

实际实现在 redis.py 和 memory.py 中。
"""

from .memory import MemcachedCache, MemoryCache
from .redis import RedisCache

__all__ = [
    "MemcachedCache",
    "MemoryCache",
    "RedisCache",
]

